/*
 * LI18NUX2K.L1/base/__wcstod_internal/__wcstod_internal.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wctype.h>
#include <errno.h>

#define __USE_ISOC99 1
#include <math.h>
#include <wchar.h>

#define STR_LENGTH 16
#define INTERNAL_FLAG 0

private char *test_locale = "LTP_1.UTF-8";
private char *c_locale = "C";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();
private void test16();

#define NO_TESTS 16
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
      {test1,   1},
      {test2,   2},
      {test3,   3},
      {test4,   4},
      {test5,   5},
      {test6,   6},
      {test7,   7},
      {test8,   8},
      {test9,   9},
      {test10, 10},
      {test11, 11},
      {test12, 12},
      {test13, 13},
      {test14, 14},
      {test15, 15},
      {test16, 16},
      {NULL,    0}
};


private void
test1()
{
  double ret, exp = 4.0;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = {0x0009, 0x000A, 0x3000, 0xFF3F, L'4', L'\0'};

  DBUG_ENTER ("test1");
  tet_infoline ("Verify that this function ignores first white-space");
  tet_infoline ("characters at nptr. The white-space characters are the ones");
  tet_infoline ("whose results applied with iswspace() is true.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("__wcstod_internal() does not skip the wide space character.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test2()
{
  double ret, exp = 0.4;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"0*4";

  DBUG_ENTER ("test2");
  tet_infoline ("Verify that this function interprets decimal-point wide");
  tet_infoline ("character according to current locale.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("Decimal point isn't intepreted according to the locale.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test3()
{
  double ret, exp = 1.4e+5;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"1.4e+5";

  DBUG_ENTER ("test3");
  tet_infoline ("When the wide characters string for conversion is");
  tet_infoline ("decimal-floating-constant, verify that this function");
  tet_infoline ("converts it to floating point number in double");
  tet_infoline ("representation.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("Cannot convert decimal-floating-constant.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test4()
{
  double ret, exp = 13.562500;	/* 0x1.b2P+3 */
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"0x1.b2P+3";

  DBUG_ENTER ("test4");
  tet_infoline ("When the wide characters string for conversion is");
  tet_infoline ("hexadecimal-floating-constant, verify that this function");
  tet_infoline ("converts it to floating point number in double");
  tet_infoline ("representation.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("Cannot convert hexadecimal-floating-constant.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test5()
{
  double ret, exp = 4.0;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"4";

  DBUG_ENTER ("test5");
  tet_infoline ("When neither an exponent part nor a decimal-point wide");
  tet_infoline ("character appears in a decimal floating point number,");
  tet_infoline ("verify that an exponent part of the appropriate type with");
  tet_infoline ("value zero is assumed to follow the last digit in the");
  tet_infoline ("string.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("Cannot convert decimal-floating-constant with neither");
      tet_infoline ("an exponent part nor a decimal-point wide character");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test6()
{
  double ret, exp = 0x8f.0p0f;
  char message[64] = "test locale is ";
  int  pathok = 0;
  wchar_t str[] = L"0x8f";

  DBUG_ENTER ("test6");
  tet_infoline ("when a binary exponent part does not appear in a");
  tet_infoline ("hexadecimal floating point number, verify that an exponent");
  tet_infoline ("part of the appropriate type with value zero is assumed to");
  tet_infoline ("follow the last digit in the string.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("cannot convert decimal-floating-constant without");
      tet_infoline ("a binary exponent part.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test7()
{
  double ret, exp = -1.23e+3;
  char message[64] = "Test locale is ";
  int  pathok = 0;
  wchar_t str[] = L"-1.23e+3";

  DBUG_ENTER ("test7");
  tet_infoline ("When the wide character string for conversion begins with a");
  tet_infoline ("minus sign, verify that the string is interpreted as");
  tet_infoline ("negated.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("cannot convert the wide character string starting");
      tet_infoline ("with minus sign");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test8()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[STR_LENGTH];
  wchar_t *endptr;

  DBUG_ENTER ("test8");
  tet_infoline ("When the correct value is outside of representable values,");
  tet_infoline ("verify that plus or minus HUGE_VAL is returned and ERANGE");
  tet_infoline ("is stored in errno.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  str[1] = L'E';  str[2] = L'+';
  ret = __wcstod_internal (str, &endptr, INTERNAL_FLAG);
  
  if (ret == HUGE_VAL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test8", "Return Value: %f", ret);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


private void
test9()
{
  double ret, exp = INFINITY;
  char message[64] = "Test locale is ";
  int  pathok = 0;
  wchar_t str[] = L"INF";

  DBUG_ENTER ("test9");
  tet_infoline ("When the wide character string for conversion is INF or");
  tet_infoline ("INFINITY, verify that it is interpreted as infinity.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstod_internal (str, NULL, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("cannot convert the wide character string starting 'INF'");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test10()
{
  double ret, exp = 0;
  char message[64] = "Test locale is ";
  int  pathok = 0;
  wchar_t str[STR_LENGTH];

  DBUG_ENTER ("test10");
  tet_infoline ("When the result underflows, verify this function returns a");
  tet_infoline ("value whose magnitude is no greater than the smallest");
  tet_infoline ("normalized positive number in the float type.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  str[1] = L'E';  str[2] = L'-';
  ret = __wcstod_internal (str, NULL, INTERNAL_FLAG);

  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "result: %f, expect: %f", ret, exp);
      tet_infoline (msg);
      tet_infoline ("cannot convert the wide character string starting 'INF'");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test11()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"4A";
  wchar_t exp = L'A', *ptr;

  DBUG_ENTER ("test11");
  tet_infoline ("When endptr is not a null pointer, verify that a pointer");
  tet_infoline ("to the character which is next to the last converted");
  tet_infoline ("character is stored in the object pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  __wcstod_internal (str, &ptr, INTERNAL_FLAG);

  if (ptr != NULL && *ptr == exp)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", *ptr, exp);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test12()
{
  int     pathok = 0;

  DBUG_ENTER ("test12");
  tet_infoline ("When the wide character string for conversion has the");
  tet_infoline ("hexadecimal form and FLT_RADIX is a power of 2, the value");
  tet_infoline ("resulting from the conversion is correctly rounded.");
  tet_infoline (" ");

  xx_rpt (UNTESTED);
  in_rpt (">> The way of rounding depends on the implementation.");
  DBUG_VOID_RETURN;
}


private void
test13()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"";
  double ret;

  DBUG_ENTER ("test13");
  tet_infoline ("When the wide character string for conversion is empty,");
  tet_infoline ("verify that return value is 0.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstod_internal (str, NULL, INTERNAL_FLAG);

  if (ret == 0.0)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "ret: %f, correct: %f", ret, 0.0);
      tet_infoline (msg);
      tet_infoline ("The return value is not correct.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test14()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"";
  wchar_t *ptr;

  DBUG_ENTER ("test14");
  tet_infoline ("When the wide character string for conversion is empty and");
  tet_infoline ("endptr is not a null pointer, verify that the value of nptr");
  tet_infoline ("is stored in the object pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  __wcstod_internal (str, &ptr, INTERNAL_FLAG);

  if (ptr != NULL && ptr == str)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", ptr, str);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test15()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"Z 4";
  double ret;

  DBUG_ENTER ("test15");
  tet_infoline ("When the wide character string for conversion does not");
  tet_infoline ("have the expected form, verify that return value is 0.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstod_internal (str, NULL, INTERNAL_FLAG);

  if (ret == 0.0)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "ret: %f, correct: %f", ret, 0.0);
      tet_infoline (msg);
      tet_infoline ("The return value is not correct.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test16()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"z 4";
  wchar_t *ptr;

  DBUG_ENTER ("test16");
  tet_infoline ("When the wide character string for conversion does not");
  tet_infoline ("have the expected form and endptr is not a null pointer,");
  tet_infoline ("verify that the value of nptr is stored in the object");
  tet_infoline ("pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  __wcstod_internal (str, &ptr, INTERNAL_FLAG);

  if (ptr != NULL && ptr == str)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", ptr, str);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
