/*
 * LI18NUX2K.L1/base/catclose/catclose.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <features.h>
#include <nl_types.h>
#include <errno.h>

/* If this variable is non-zero, something is wrong at prepare(). */
private int fault = 0;

private void prepare();
private void cleanup();
private void test1();
private void test2();

#define NO_TESTS 2
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = prepare;
public 	void  	(*clnfunc)()     = cleanup;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	NULL, 0
};

/* Current working directory. */
private char   *cwd;
private size_t cwdlen;

/* Test directories. */
private char *testdir[] = {"testpath0", "testpath1"};
private size_t testdir_num;
private char *nulldir = "null";
private char *nullpath;

/* Locales for testing. */
private char *locale[] = {"C", "LTP_1.UTF-8"};
private size_t locale_num;

/* Message file name (without suffix). */
private char *basename = "test";

/* Message Catalog generator. */
private char *generator = "gencat";

/* One of messages. */
private int set_id = 1;
private int message_id = 1;
private char ***message;

/* Default messages. */
private char *default_message = "default";

/* Preparation for testing. */
private void
prepare()
{
  char   *cwdtemp;
  size_t cwdtemplen = 8;
  size_t i, j;

  /* get current working directory. */
  do {
    cwdtemplen += 8;
    cwdtemp = calloc (cwdtemplen, sizeof(char));
  } while (getcwd (cwdtemp, cwdtemplen) == NULL);

  cwdlen = strlen (cwdtemp);
  cwd = calloc (cwdlen + 1, sizeof(char));
  memcpy (cwd, cwdtemp, cwdlen);
  free (cwdtemp);

  testdir_num = sizeof(testdir) / sizeof(testdir[0]);
  locale_num  = sizeof(locale)  / sizeof(locale[0]);

  message = calloc (testdir_num, sizeof(char **));

  for (i = 0; i < testdir_num; i++)
    {
      message[i] = calloc (locale_num, sizeof (char *));
      for (j = 0; j < locale_num; j++)
	{
	  FILE *fp;
	  char *msgfile, *catfile;
	  char *cmd;

	  msgfile = calloc (cwdlen + 4 + strlen (testdir[i]) + 1
			    + strlen (locale[j]) + 1 + strlen (basename) + 5
			    , sizeof(char));
	  sprintf (msgfile, "%s/../%s/%s/%s.msg"
		    , cwd, testdir[i], locale[j], basename);

	  catfile = calloc (cwdlen + 4 + strlen (testdir[i]) + 1
			    + strlen (locale[j]) + 1 + strlen (basename) + 5
			    , sizeof(char));
	  sprintf (catfile, "%s/../%s/%s/%s.cat"
		    , cwd, testdir[i], locale[j], basename);

	  /* Read one of the messages in each set. */
	  if ((fp = fopen (msgfile, "r")) == NULL)
	    {
	      fault++;
	      DBUG_PRINTF ("prepare", "The following file is not found.\n"
			   "> %s", msgfile);
	      return;
	    }
	  else
	    {
	      char line[BUFSIZ], comp[BUFSIZ], temp[BUFSIZ], *p;

	      sprintf (comp, "$set %d\n", set_id);
	      DBUG_PRINTF ("prepare", "> comp: %s", comp);

	      /* Find the selected set. */
	      while (1)
		{
		  if (fgets (line, BUFSIZ, fp) == NULL)
		    {
		      fault++;
		      break;
		    }
		  if (strcmp (line, comp) == 0)
		    break;
		}

	      /* Find the setlected message. */
	      memset (comp, '\0', BUFSIZ);
	      sprintf (comp, "%d", message_id);
	      while (1)
		{
		  if (fgets (line, BUFSIZ, fp) == NULL)
		    {
		      fault++;
		      break;
		    }
		  if (strncmp (line, comp, strlen (comp)) == 0)
		    break;
		}

	      if (!fault)
		{
		  if ((p = strchr (line, '"')) == NULL)
		    {
		      fault++;
		      break;
		    }
		  strcpy (temp, p + 1);

		  if ((p = strrchr (temp, '"')) == NULL)
		    {
		      fault++;
		      break;
		    }
		  *p = '\0';

		  message[i][j] = calloc ((strlen (temp) + 1), sizeof (char));
		  strcpy (message[i][j], temp);

		  DBUG_PRINTF ("prepare", "message: %s", message[i][j]);
		}

	      fclose (fp);
	    }

	  /* Generate message catalog file. */
	  setenv ("LC_ALL", locale[j], 1);
	  cmd = calloc (strlen (generator) + 1 + strlen (catfile) + 1 +
			strlen (msgfile) + 1, sizeof (char));
	  sprintf (cmd, "%s %s %s", generator, catfile, msgfile);
	  system (cmd);

	  if ((fp = fopen (catfile, "r")) == NULL)
	    {
	      fault++;
	      DBUG_PRINTF ("prepare", "The catalog file is not cleated.\n"
			   "> %s", catfile);
	      return;
	    }
	  else
	    fclose (fp);
	  
	  free (msgfile);
	  free (catfile);
	  free (cmd);
	}
    }

  /* Define null path. */
  nullpath = calloc (cwdlen + 4 + strlen (nulldir) + 1, sizeof(char));
  sprintf (nullpath, "%s/../%s", cwd, nulldir);
  return;
}

/* Crean up something for testing. */
private void
cleanup()
{
  int i, j;

  for (i = 0; i < testdir_num; i++)
    {
      for (j =0; j < locale_num; j++)
	free (message[i][j]);
      free (message[i]);
    }
  free (message);

  free (nullpath);
  free (cwd);
}


/* If the call is successful, verify this function returns 0. */
private void
test1()
{
  int  ret;
  nl_catd catalog;
  char *catfile;
  int  pathok = 0;
  char *desc = "If the call is successful, verify this function returns 0.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (fault)
    {
      in_rpt ("The preparation is something wrong.");
      DBUG_VOID_RETURN;
    }

  catfile = calloc (cwdlen + 4 + strlen (testdir[0]) + 1 + strlen (locale[0])
		    + 1 + strlen (basename) + 5, sizeof(char));
  sprintf (catfile, "%s/../%s/%s/%s.cat", cwd, testdir[0], locale[0], basename);

  globok = 1;

  if ((catalog = catopen (catfile, NL_CAT_LOCALE)) == (nl_catd)-1)
    {
      xx_rpt (FAILURE);
      in_rpt ("The catopen is not called correctly.");
    }
  free (catfile);

  if ((ret = catclose (catalog)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the call is unsuccessful for any reason, verify this function returns
   -1, and errno is set to indicate the error. */
private void
test2()
{
  int  ret;
  nl_catd catalog;
  char *catfile;
  int  pathok = 0;
  char *desc = "If the call is unsuccessful for any reason, verify this "
	       "function returns -1, and errno is set to indicate the error.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  xx_rpt (UNTESTED);
  in_rpt ("--");
  in_rpt ("This assrtion is not tested.");
  in_rpt ("We have no idea to produce these situation.");

  /* The following test is wrong. */

  /*
  if (fault)
    {
      in_rpt ("The preparation is something wrong.");
      DBUG_VOID_RETURN;
    }

  catfile = calloc (cwdlen + 4 + strlen (testdir[0]) + 1 + strlen (locale[0])
		    + 1 + strlen (basename) + 5, sizeof(char));
  sprintf (catfile, "%s/../%s/%s/%s.cat", cwd, testdir[0], locale[0], basename);

  globok = 1;

  if ((catalog = catopen (catfile, NL_CAT_LOCALE)) == (nl_catd)-1)
    {
      xx_rpt (FAILURE);
      in_rpt ("The catopen is not called correctly.");
    }
  free (catfile);

  if ((ret = catclose (catalog)) != 0)
    {
      xx_rpt (FAILURE);
      in_rpt ("The catalog is not closed correctly.");
    }

  if ((ret = catclose (catalog)) == -1)
    if (errno == EBADF)
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The errno is not correct.");
	DBUG_PRINTF ("test2", "errno: %d", errno);
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test2", "Return value: %d", ret);
    }

  PATH_XS_RPT(1);
   */

  DBUG_VOID_RETURN;
}
