/*
 * LI18NUX2K.L1/base/getwchar/getwchar.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <errno.h>
#include <locale.h>
#include <wchar.h>

#define	CHAR_NUM  3
#define INVALID_BYTE_LEN  3

private char *test_locale = "LTP_1.UTF-8";

private wint_t wchara[CHAR_NUM] = {L'A', 0x00C4, 0x3042};
private char *chara[CHAR_NUM]   = {"A",  "Ä",    "あ"};
private char invalid_chara[INVALID_BYTE_LEN] = {0xe3, 0x21, 0x00}; 

FILE    *fp;
private void reopen_stdin();
private void close_file();

private void test1();
private void test2();
private void test3();

#define NO_TESTS 3
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = reopen_stdin;
public 	void  	(*clnfunc)()     = close_file;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	NULL, 0
};


/* Change standard input to an usual file. */
private void
reopen_stdin()
{
  int i;

  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      return;
    }

  for (i = 0; i < CHAR_NUM; i++)
    fprintf (fp, "%s", chara[i]);
  fprintf (fp, "%s", chara[0]);
  fwrite (invalid_chara, sizeof(char), INVALID_BYTE_LEN, fp);
  
  fclose (fp);

  return;
}

/* Close fp */
private void
close_file()
{
  if (fp != NULL)
    fclose (fp);
  return;
}


/* If the byte sequence given via standard input is valid as character on the
   current locale, verify this function returns the wide character which is
   equivalent to the multibyte character. */
private void
test1()
{
  int i;
  wint_t  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the byte sequence given via standard input is valid as "
	       "character on the current locale, verify this function returns "
	       "the wide character which is equivalent to the multibyte "
	       "character.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  fp = freopen ("testfile", "r", stdin);
  if (fp == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  for (i = 0; i < CHAR_NUM; i++)
    {
      if ((ret = getwchar()) == wchara[i])
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The return value is not correct.");
	  DBUG_PRINTF ("test1", "Return Value: %lc", ret);
	}
    }

  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}


/* If the byte sequence given via standard input is invalid as character on
   the current locale, verify this function returns WEOF and set errno
   EILSEQ. */
private void
test2()
{
  wint_t  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the byte sequence given via standard input is invalid as "
	       "character on the current locale, verify this function "
	       "returns WEOF and set errno EILSEQ.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  fp = freopen ("testfile", "r", stdin);
  if (fp == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = getwchar();

  if (fwide(stdin, 0) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* After executing this function correctly, the type of standard input is
   changed to wide character oriented. */
private void
test3()
{
  wint_t  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "After executing this function correctly, the type of standard "
	       "input is changed to wide character oriented.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  fp = freopen ("testfile", "r", stdin);
  if (fp == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  errno = 0;

  if ((ret = getwchar()) == WEOF)
    if (errno == EILSEQ)
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The errno is not correct.");
        DBUG_PRINTF ("test3", "errno: %d", errno);
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test3", "Return Value: %lc", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
