/*
 * LI18NUX2K.L1/base/strfmon/strfmon.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <wchar.h>
#include <wctype.h>
#include <monetary.h>
#include <errno.h>

#define BUFFER_SIZE 255

private char *test_locale = "LTP_1.UTF-8";

private char *formats[] = {"%=*#5n",		"%=0#5n",		"%#5n",
			   "%^n",		"%n",			"%+n",
			   "%+n",		"%(n",			"%(n",
			   "%n",		"%n",			"%!n",
                           "%-12n",		"%10n",			"",
			   "%#5n",		"%#6n",			"%.0n",
			   "%.1n",		"%i",			"%n",
			   "%%"};
private double values[] = {123.0,		456.0,			789.0,
			   12345.0,		12345.0,		678.0,
			   -901.0,		234.0,			-567.0,
			   890.0,		-123.0,			456.0,
                           789.0,		12.0,			0.0,
			   3.0,			45678.0,		90.12,
			   3.45,		678.0,			901.0,
			   0.0};

private char answers[][32] = {{0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '*', '*', '*', '1', '2', '|', '3', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '0', '0', '0', '4', '5', '|', '6', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       ' ', ' ', ' ', '7', '8', '|', '9', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '1', '2', '3', '4', '5', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '1', '2', '|', '3', '4', '|', '5', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '6', '7', '|', '8', '\0'},
			      {0xE2, 0x96, 0xBD, 0xE2, 0x99, 0xAA,
			       '9', '0', '|', '1', '\0'},
			      {0xE2, 0x99, 0xAA,
			       '2', '3', '|', '4', '\0'},
			      {'(', 0xE2, 0x99, 0xAA,
			       '5', '6', '|', '7', ')', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '8', '9', '|', '0', '\0'},
			      {0xE2, 0x96, 0xBD, 0xE2, 0x99, 0xAA,
			       '1', '2', '|', '3', '\0'},
			      {0xE2, 0x96, 0xB2, 
			       '4', '5', '|', '6', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '7', '8', '|', '9',  ' ', ' ', '\0'},
			      {' ', 0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '1', '|', '2', '\0'},
			      {'\0'}, 
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       ' ', ' ', ' ', ' ', ' ', ' ', '3', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       ' ', '4', '5', '|', '6', '7', '|', '8', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '9', '|', '0', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '3', '*', '5', '\0'},
			      {0xE2, 0x96, 0xB2, 'F', 'U', 'N', ' ',
			       '6', '7', '|', '8', '\0'},
			      {0xE2, 0x96, 0xB2, 0xE2, 0x99, 0xAA,
			       '9', '0', '|', '1', '\0'},
			      {'%', '\0'},
			     };

private int testloop(int number, char* message);
private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();
private void test16();
private void test17();
private void test18();
private void test19();
private void test20();

#define NO_TESTS 20
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	test4,  4,
	test5,  5,
	test6,  6,
	test7,  7,
	test8,  8,
	test9,  9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	NULL, 0
};

private int testloop(int number, char* message)
{
  ssize_t rval;
  char buf[BUFFER_SIZE];

  rval = strfmon(buf, BUFFER_SIZE, formats[number], values[number]);

  if (rval == strlen(answers[number]))
    {
      if (strcmp(buf, answers[number]) == 0)
        return 1;
      else
        {
            sprintf(message, "\"%s\" is incorrectly evaluated.)",
                    formats[number]);
        }
    }
  else
    {
        sprintf(message, "Incorrect return value for \"%s\".\n"
			 "Return Value:%s\n"
			 "Correct One :%s", formats[number], buf, answers[number]);
    }
  return 0;
}

/* If a flag =c is given, verify this function uses c as the numeric fill
   character.  */
private void
test1()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag =c is given, verify this function uses c as "
                 "the numeric fill character.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (!testloop(0, buf))
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  if (testloop(1, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag =c is not given, verify this function uses space character as
   the numeric fill character.  */
private void
test2()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag =c is not given, verify this function uses "
                 "space character as the numeric fill character.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(2, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag ^ is given, verify this function do not format the currency
   amount with grouping characters.  */
private void
test3()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag ^ is given, verify this function do not format "
                 "the currency amount with grouping characters.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(3, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag ^ is not given, verify this function insert the grouping
   characters if defined for the current locale.  */
private void
test4()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag ^ is not given, verify this function insert the "
                 "grouping characters if defined for the current locale.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(4, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag + is given, verify this function sets the locale's equivalent
   of + and - for the style of representing positive and negative currency
   amounts.  */
private void
test5()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag + is given, verify this function sets the "
                 "locale's equivalent of + and - for the style of "
                 "representing positive and negative currency amounts.";

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (!testloop(5, buf))
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  if (testloop(6, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag ( is given, verify this function sets negative amounts are
   enclosed within parentheses for the style of representing positive and
   negative currency amounts.  */
private void
test6()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag ( is given, verify this function sets negative "
                 "amounts are enclosed within parentheses for the style of "
                 "representing positive and negative currency amounts.";

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (!testloop(7, buf))
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  if (testloop(8, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If neither + nor ( is given, verify this function sets the locale's
   equivalent of + and - for the style of representing positive and negative
   currency amounts.  */
private void
test7()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If neither + nor ( is given, verify this function sets "
                 "the locale's equivalent of + and - for the style of "
                 "representing positive and negative currency amounts.";

  DBUG_ENTER ("test7");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (!testloop(9, buf))
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  if (testloop(10, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag ! is given, verify this function suppresses the currency symbol
   from the output conversion.  */
private void
test8()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag ! is given, verify this function suppresses "
                 "the currency symbol from the output conversion.";

  DBUG_ENTER ("test8");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(11, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag - is given, verify this function all fields are left-justified
   (padded to the right) rather than right-justified.  */
private void
test9()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag - is given, verify this function all fields "
                 "are left-justified (padded to the right) rather than "
                 "right-justified.";

  DBUG_ENTER ("test9");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(12, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* Verify this function replaces conversion specifier w with decimal digit
   string which shows minimum field width in bytes in which the result of
   the conversion is right-justified (or left-justified if the flag - is
   specified).  */
private void
test10()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "Verify this function replaces conversion specifier w "
                 "with decimal digit string which shows minimum field "
                 "width in bytes in which the result of the conversion is "
                 "right-justified (or left-justified if the flag - is "
                 "specified).";

  DBUG_ENTER ("test10");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(13, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag w is not defined, The field width is 0.  */
private void
test11()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag w is not defined, The field width is 0.";

  DBUG_ENTER ("test11");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(14, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag #n (n is a maximum number of digits) is given, verify this
   function formats to the left of the radix character.   */
private void
test12()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag #n (n is a maximum number of digits) is given, "
                 "verify this function formats to the left of the radix "
                 "character.";

  DBUG_ENTER ("test12");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(15, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag #n (n is a maximum number of digits) is given, and grouping has
   not been suppressed with the ^ flag, grouping separators are inserted
   before the fill characters (if any) are added.  */
private void
test13()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag #n (n is a maximum number of digits) is given, "
                 "and grouping has not been suppressed with the ^ flag, "
                 "grouping separators are inserted before the fill "
                 "characters (if any) are added.";

  DBUG_ENTER ("test13");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(16, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag .n (n is the number of digits after the radix character) is
   given and n is 0, no radix character appears.  */
private void
test14()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag .n (n is the number of digits after the radix "
                 "character) is given and n is 0, no radix character appears.";

  DBUG_ENTER ("test14");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(17, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If a flag .n (n is the number of digits after the radix character) is
   given and the value of the right precision n is non-zero, the amount
   being formatted is rounded to the specified number of digits prior to
   formatting.  */
private void
test15()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If a flag .n (n is the number of digits after the radix "
                 "character) is given and the value of the right precision "
                 "n is non-zero, the amount being formatted is rounded to "
                 "the specified number of digits prior to formatting.";

  DBUG_ENTER ("test15");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(18, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* Verify this function replaces conversion specifier i with the double
   argument is formatted according to the locale's international currency
   format.  */
private void
test16()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "Verify this function replaces conversion specifier i "
                 "with the double argument is formatted according to the "
                 "locale's international currency format.";

  DBUG_ENTER ("test16");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(19, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* Verify this function replaces conversion specifier n with the double
   argument is formatted according to the locale's national currency
   format.  */
private void
test17()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "Verify this function replaces conversion specifier n "
                 "with the double argument is formatted according to the "
                 "locale's national currency format.";

  DBUG_ENTER ("test17");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(20, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* Verify this function replaces conversion specifier % with a %.  */
private void
test18()
{
  char   buf[BUFFER_SIZE];
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "Verify this function replaces conversion specifier % "
                 "with a %.";

  DBUG_ENTER ("test18");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if (testloop(21, buf))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt (buf);
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If the call is successful and the total number of resulting bytes
   including the terminating null byte is not more than maxsize, verify
   this function returns the number of bytes replaced into the array
   pointed to by s, not including the terminating null byte.  */
private void
test19()
{
  char   buf[BUFFER_SIZE];
  ssize_t rval;
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If the call is successful and the total number of "
                 "resulting bytes including the terminating null byte is "
                 "not more than maxsize, verify this function returns the "
                 "number of bytes replaced into the array pointed to by s, "
                 "not including the terminating null byte.";

  DBUG_ENTER ("test19");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  rval = strfmon(buf, BUFFER_SIZE, "%n", 123.0);

  if (rval == 10)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Incorrect return value.");
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/* If the call is unsuccessful, verify this function returns -1 and errno
   is set to indicate the error.  */
private void
test20()
{
  char   buf[BUFFER_SIZE];
  ssize_t rval;
  char   message[64] = "Test Locale is ";
  int    pathok = 0;
  char *desc = "If the call is unsuccessful, verify this function "
                 "returns -1 and errno is set to indicate the error.";

  DBUG_ENTER ("test20");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  rval = strfmon(buf, 1, "%n", 123.0);

  if (rval == -1)
    {
      if (errno == E2BIG)
        PATH_TRACE;
      else
        {
          xx_rpt (FAILURE);
          in_rpt ("Incorrect errno.");
          DBUG_VOID_RETURN;
        }
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Incorrect return value for an error.");
      DBUG_VOID_RETURN;
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
