/*
 * LI18NUX2K.L1/base/textdomain/textdomain.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <langinfo.h>
#include <libintl.h>
#include <iconv.h>

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();

#define NO_TESTS 7
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	test5, 5,
	test6, 6,
	test7, 7,
	NULL, 0
};

private void
test1()
{
  int	  pathok = 0;
  char	  *correct = "2£テスト";
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;

  DBUG_ENTER ("test1");
  tet_infoline ("When the domainname is not NULL nor the pointer to null");
  tet_infoline ("string and this function called successfully, verify that");
  tet_infoline ("the message outputted by gettext family searches the message");
  tet_infoline ("from domain whose name is domainname.");  
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain (dommainname, dirname);
  textdomain (dommainname);

  if (strcmp (correct, gettext ("good bye")) == 0)
    PATH_TRACE;
  else
    {
      char msg[1024];

      xx_rpt (FAILURE);
      in_rpt ("gettext got worng text.");
      in_rpt ("So bindtextdomain, textdomain or gettext works wrongly.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test2()
{
  int	  pathok = 0;
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;

  DBUG_ENTER ("test2");
  tet_infoline ("When the domainname is not NULL nor the pointer to null");
  tet_infoline ("string and this function called successfully, verify that");
  tet_infoline ("this functions returns domainname.");
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain (dommainname, dirname);

  if (strcmp (dommainname, textdomain (dommainname)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test3()
{
  int	  pathok = 0;
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;
  char	  *correct = "2£テスト";

  DBUG_ENTER ("test3");
  tet_infoline ("When the domainname is NULL and this function called");
  tet_infoline ("successfully, verify that this function returns the name of");
  tet_infoline ("current domain.");
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain (dommainname, dirname);
  textdomain (dommainname);

  if (strcmp (dommainname, textdomain (NULL)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test4()
{
  int	  pathok = 0;
  char	  *correct = "§＠⇔";
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;

  DBUG_ENTER ("test4");
  tet_infoline ("When the domainname is the pointer to null string and this");
  tet_infoline ("function called successfully, verify that the current domain");
  tet_infoline ("is changed to `messages'.");
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain ("messages", dirname);
  bindtextdomain (dommainname, dirname);
  textdomain (dommainname);
  textdomain ("");

  if (strcmp (correct, gettext ("good bye")) == 0)
    PATH_TRACE;
  else
    {
      char msg[1024];

      xx_rpt (FAILURE);
      in_rpt ("gettext got worng text.");
      in_rpt ("So bindtextdomain, textdomain or gettext works wrongly.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test5()
{
  int	  pathok = 0;

  DBUG_ENTER ("test5");
  tet_infoline ("When this function fails, verify that the return value is NULL.");
  tet_infoline (" ");

  xx_rpt (UNTESTED);
  in_rpt ("It is hard to let textdomain fail.");

  DBUG_VOID_RETURN;
}

private void
test6()
{
  int	  pathok = 0;
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;

  DBUG_ENTER ("test6");
  tet_infoline ("Verify that the domain defined by this function is remained");
  tet_infoline ("even if gettext family is called.");
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain (dommainname, dirname);
  gettext ("hello");

  if (strcmp (dommainname, textdomain (dommainname)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test7()
{
  int	  pathok = 0;
  char	  *dommainname ="foo";
  char	  *dirname = NULL;
  size_t  dirnamelen = 16;

  DBUG_ENTER ("test7");
  tet_infoline ("Verify that the domain defined by this function is remained");
  tet_infoline ("even if setlocale is called.");
  tet_infoline (" ");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;

  for (;;)
    {
      if ((dirname = (char *)realloc (dirname, dirnamelen + 8)) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("There is not enough memory.");
	  DBUG_VOID_RETURN;
	}
      memset (dirname, '\0', dirnamelen + 8);

      if ((getcwd (dirname, dirnamelen)) != NULL)
	{
	  char *p;
	  p = strrchr (dirname, '/');
	  *p = '\0';
	  break;
	}

      else
	dirnamelen += 16;
    }


  strcat (dirname, "/test1");

  bindtextdomain (dommainname, dirname);
  setlocale (LC_ALL, "POSIX");

  if (strcmp (dommainname, textdomain (dommainname)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
