/*
 * LI18NUX2K.L1/base/vfwscanf/vfwscanf.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <locale.h>

#define __USE_ISOC99 1
#include <stdarg.h>
#include <wchar.h>

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();
private void test16();
private void test17();
private void test18();
private void test19();
private void test20();
private void test21();
private void test22();
private void test23();
private void test24();
private void test25();

#define NO_TESTS 25
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	{test1,   1},
	{test2,   2},
	{test3,   3},
	{test4,   4},
	{test5,   5},
	{test6,   6},
	{test7,   7},
	{test8,   8},
	{test9,   9},
	{test10, 10},
	{test11, 11},
	{test12, 12},
	{test13, 13},
	{test14, 14},
	{test15, 15},
	{test16, 16},
	{test17, 17},
	{test18, 18},
	{test19, 19},
	{test20, 20},
	{test21, 21},
	{test22, 22},
	{test23, 23},
	{test24, 24},
	{test25, 25},
	{NULL,    0}
};

/* Like wscanf with using vwscanf(). */
private int
va_fwscanf (FILE *stream, wchar_t *format, ...)
{
  va_list argp;
  int ret;

  DBUG_ENTER ("va_fwscanf");
  va_start (argp, format);
  ret = vfwscanf (stream, format, argp);
  va_end (argp);

  DBUG_RETURN (ret);
}

/* Compare two arrays of wide characters.
   If two array is the same, this function returns 1, others returns 0. */
private int
is_wcs_same (const wchar_t *ws1, const wchar_t *ws2, size_t n)
{
  int i;

  DBUG_ENTER ("is_wcs_same");
  for (i = 0; i < n; i++)
    if (ws1[i] != ws2[i])
      DBUG_RETURN (0);

  DBUG_RETURN (1);
}


private void
test1()
{
  wchar_t wc;
  int ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test1");
  tet_infoline ("When a directive is composed of white-space wide characters,");
  tet_infoline ("verify that this function reads input up to the first");
  tet_infoline ("non-white-space wide character (which remains unread), or");
  tet_infoline ("it reads input until no more wide characters can be read.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile1", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile1'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "  a \t\t ");
  fclose (fp);

  if ((fp = fopen ("testfile1", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile1'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"  %lc", &wc);
  if (wc == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Did not skip up to the first wide character which is not ");
      in_rpt ("white space wide character.");
      DBUG_PRINTF ("test1", "Character: %lc", wc);
    }

  ret = va_fwscanf (fp, L" \t%lc", &wc);
  if (ret == EOF)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Did not skip no more valid input can be read.");
      DBUG_PRINTF ("test1", "Character: %lc", wc);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


private void
test2()
{
  int ret, n;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test2");
  tet_infoline ("When a directive is an ordinary wide character (i.e. except");
  tet_infoline ("for %, and white-space wide character) and the wide");
  tet_infoline ("character is equivalent to the wide character which would");
  tet_infoline ("be read from the stream, verify that this function");
  tet_infoline ("read the next wide character from the stream.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile2", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile2'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "a9bc8");
  fclose (fp);

  if ((fp = fopen ("testfile2", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile2'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"a%d", &n);
  if (n == 9)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The next character is not read.");
      DBUG_PRINTF ("test2", "number: %d", n);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test3()
{
  int ret, n;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test2");
  tet_infoline ("When a directive is an ordinary wide character (i.e. except");
  tet_infoline ("for %, and white-space wide character) and the wide");
  tet_infoline ("character is not equivalent to the wide character which");
  tet_infoline ("would be read from the stream, verify that the");
  tet_infoline ("directive fails and the next wide character on the standard");
  tet_infoline ("input remains unread.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile3", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile3'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "bc8");
  fclose (fp);

  if ((fp = fopen ("testfile3", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile3'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"bb%d", &n);
  if (ret < 1)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Subsequent wide characters does not remain unread.");
      DBUG_PRINTF ("test2", "Return Value: %d", ret);
    }

  ret = va_fwscanf (fp, L"c%d", &n);
  if (n == 8)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The next character is not read.");
      DBUG_PRINTF ("test2", "number: %d", n);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}

private void
test4()
{
  wchar_t wc;
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test4");
  tet_infoline ("When a conversion specification has `*', verify that the");
  tet_infoline ("result of conversion is not stored any arguments.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile4", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile4'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "ab");
  fclose (fp);

  if ((fp = fopen ("testfile4", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile4'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%*lc%lc", &wc);
  if (wc == L'b')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("`*' is interpreted wrongly.");
      DBUG_PRINTF ("test4", "Return Value: %d", ret);
      DBUG_PRINTF ("test4", "Read: %lc", wc);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test5()
{
  wchar_t wcs[] = {L'\0', L'\0', L'\0', L'\0', L'\0',
		  L'\0', L'\0', L'\0', L'\0', L'\0'};
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test5");
  tet_infoline ("When a conversion specification has decimal wide characters,");
  tet_infoline ("verify that this function interprets the wide characters as");
  tet_infoline ("the field width.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile5", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile5'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "123456789");
  fclose (fp);

  if ((fp = fopen ("testfile5", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile5'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%5lc", wcs);
  if (is_wcs_same (wcs, L"12345\0", 6))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The field width is interpreted wrongly.");
      DBUG_PRINTF ("test5", "Return Value: %d", ret);
      DBUG_PRINTF ("test5", "Read: %ls", wcs);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test6()
{
  wchar_t wcs[] = L"   123";
  char    str[] =  "   123";
  long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test6");
  tet_infoline ("When a conversion wide character is `d', verify that the");
  tet_infoline ("conversion result is the same as the results of wcstol()");
  tet_infoline ("with the value 10 for the base argument.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile6", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile6'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile6", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile6'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%ld", &n1);
  n2 = wcstol(wcs, NULL, 10);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `d' is not interpreted correctly");
      in_rpt ("(Or wcstol() is something wrong.)");
      DBUG_PRINTF ("test6", "Return Value: %d", ret);
      DBUG_PRINTF ("test6", "va_fwscanf(): %ld, wcstol(): %ld", n1, n2);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


#define TEST7_STR_NUM	4
private void
test7()
{
  wchar_t *wcs[] = {L"   077", L" 123", L"  0xff", L"  0XFFFF"};
  char    *str[] = { "   077",  " 123",  "  0xff",  "  0xFFFF"};
  int  i, ret;
  long n1, n2;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test7");
  tet_infoline ("When a conversion wide character is `i', verify that the");
  tet_infoline ("conversion result is the same as the results of wcstol()");
  tet_infoline ("with the value 0 for the base argument.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }
  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile7", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile7'");
      DBUG_VOID_RETURN;
    }
  for (i = 0; i < TEST7_STR_NUM; i++)
    fprintf (fp, "%s\n", str[i]);
  fclose (fp);

  if ((fp = fopen ("testfile7", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile7'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < TEST7_STR_NUM; i++)
    {
      ret = va_fwscanf (fp, L"%li", &n1);
      n2 = wcstol(wcs[i], NULL, 0);

      if (n1 == n2)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Conversion wide character `i' is not interpreted correctly");
	  in_rpt ("(Or wcstol() is something wrong.)");
	  DBUG_PRINTF ("test7", "Return Value: %d", ret);
	  DBUG_PRINTF ("test7", "va_fwscanf(): %li, wcstol(): %li", n1, n2);
	}
    }

  fclose (fp);

  PATH_XS_RPT(TEST7_STR_NUM);
  DBUG_VOID_RETURN;
}


private void
test8()
{
  wchar_t wcs[] = L"   779";
  char    str[] =  "   779";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test8");
  tet_infoline ("When a conversion wide character is `o', verify that the");
  tet_infoline ("conversion result is the same as the results of wcstoul()");
  tet_infoline ("tet_with the value 8 for the base argument.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile8", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile8'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile8", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile8'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%lo", &n1);
  n2 = wcstoul(wcs, NULL, 8);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `o' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test8", "Return Value: %d", ret);
      DBUG_PRINTF ("test8", "va_fwscanf(): %lo, wcstoul(): %lo", n1, n2);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test9()
{
  wchar_t wcs[] = L"   2147483648";
  char    str[] =  "   2147483648 ";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test9");
  tet_infoline ("When a conversion wide character is `u', verify that the");
  tet_infoline ("conversion result is the same as the results of wcstoul()");
  tet_infoline ("with the value 10 for the base argument.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile9", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile9'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile9", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile9'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%lu", &n1);
  n2  = wcstoul(wcs, NULL, 10);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `u' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test9", "Return Value: %d", ret);
      DBUG_PRINTF ("test9", "va_fwscanf(): %lu, wcstoul(): %lu", n1, n2);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test10()
{
  wchar_t wcs[] = L"   ABCDEFG";
  char    str[] =  "   ABCDEFG ";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test10");
  tet_infoline ("When a conversion wide character is `x' and `X', verify");
  tet_infoline ("that the conversion result is the same as the results of");
  tet_infoline ("wcstoul() with the value 16 for the base argument.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile10", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile10'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile10", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile10'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%lx", &n1);
  n2 = wcstoul(wcs, NULL, 16);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `x' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test10", "Return Value: %d", ret);
      DBUG_PRINTF ("test10", "va_fwscanf(): %lx, wcstoul(): %lx", n1, n2);
    }
  rewind (fp);

  ret = va_fwscanf (fp, L"%lX", &n1);
  n2 = wcstoul(wcs, NULL, 16);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `X' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test10", "Return Value: %d", ret);
      DBUG_PRINTF ("test10", "va_fwscanf(): %lX, wcstoul(): %lX", n1, n2);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


private void
test11()
{
  wchar_t *conv[] = {L"%le", L"%lE", L"%lf", L"%lg", L"%lG"};
  wchar_t wcs[] = L"   9*9*9";
  char    str[] =  "   9*9*9";
  double  d1, d2;
  int  i, ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test11");
  tet_infoline ("When a conversion wide character is `a', `A', `e', `E',");
  tet_infoline ("`f', `F', 'g' and `G', verify the conversion result is the");
  tet_infoline ("same as the results of wcstod(). Especially check the radix");
  tet_infoline ("character according to the current locale is interpreted");
  tet_infoline ("correctly.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile11", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile11'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

 for (i = 0; i < sizeof (conv) / sizeof (conv[0]); i++)
   {
     if ((fp = fopen ("testfile11", "r")) == NULL)
       {
	 xx_rpt (FAILURE);
	 in_rpt ("Cannot open 'testfile11'");
	 DBUG_VOID_RETURN;
       }

     ret = va_fwscanf (fp, conv[i], &d1);
     d2 = wcstod(wcs, NULL);

     if (d1 == d2)
       PATH_TRACE;
     else
       {
	 xx_rpt (FAILURE);
	 in_rpt ("Conversion wide character is not interpreted correctly");
	 in_rpt ("(Or wcstoul() is something wrong.)");
	 DBUG_PRINTF ("test11", "Conversion character: %ls", conv[i]);
	 DBUG_PRINTF ("test11", "Return Value: %d", ret);
	 DBUG_PRINTF ("test11", "va_fwscanf(): %lf, wcstoul(): %lf", d1, d2);
       }
     fclose (fp);
   }

  PATH_XS_RPT(sizeof(conv) / sizeof(conv[0]));
  DBUG_VOID_RETURN;
}


private void
test12()
{
  wchar_t wcs[] = {L'A', 0x00C4, 0x3042, L'\0'}, *p;
  const wchar_t **pp;
  char *str1 = "AÄあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test12");
  tet_infoline ("When a conversion wide character is `c' and conversion");
  tet_infoline ("specification does not have `l', verify that this function");
  tet_infoline ("reads a sequence of the wide characters whose length is");
  tet_infoline ("specified by the field width and this function stores the");
  tet_infoline ("multibyte character sequence which are made of the sequence");
  tet_infoline ("of the wide characters by wcsrtombs() with initial");
  tet_infoline ("conversion state.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile12", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile12'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = fopen ("testfile12", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile12'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%4c", &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, bytelen) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test13()
{
  wchar_t wcs1[] = {L'A', 0x00C4, 0x3042, L'\0'}, wcs2[BUFSIZ];
  char *str = "AÄあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test13");
  tet_infoline ("When a conversion wide character is `c' and conversion");
  tet_infoline ("specification has `l', verify that this function reads a");
  tet_infoline ("sequence of the wide characters of whose number is");
  tet_infoline ("specified by the field width and this function stores them.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile13", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile13'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile13", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile13'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%4lc", &wcs2);

  if (is_wcs_same (wcs1, wcs2, 4))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test14()
{
  wchar_t wcs[] = {L'A', 0x00C4, 0x3042, L'\0'}, *p;
  const wchar_t **pp;
  char *str1 = "   AÄあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test14");
  tet_infoline ("When a conversion wide character is `s' and conversion");
  tet_infoline ("specification does not have `l', verify that this function");
  tet_infoline ("reads a wide character string and this function stores the");
  tet_infoline ("multibyte character string which that are made of the wide");
  tet_infoline ("character string by wcsrtombs() with initial conversion");
  tet_infoline ("state. Especially check the null character is stood at the");
  tet_infoline ("end of the multibyte character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile14", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile14'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = fopen ("testfile14", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile14'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%s", &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, bytelen) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);
  
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test15()
{
  wchar_t wcs1[] = {L'A', 0x00C4, 0x3042, L'\0'}, wcs2[BUFSIZ];
  char *str = "AÄあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test15");
  tet_infoline ("When a conversion wide character is `s' and conversion");
  tet_infoline ("specification has `l', verify that this function reads a");
  tet_infoline ("wide character string and this function stores them.");
  tet_infoline ("Especially check the null wide character is stood at the");
  tet_infoline ("end of wide character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile15", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile15'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile15", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile15'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, L"%ls", &wcs2);

  if (is_wcs_same (wcs1, wcs2, 4))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test16()
{
  wchar_t wcs[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'};
  wchar_t conv[] = {L'%', L'[', L'A', 0x00C4, 0x3042, L']', L'\0'};
  wchar_t *p;
  const wchar_t **pp;
  char *str1 = "AÄAああいあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test16");
  tet_infoline ("When a conversion wide characters are `[...]', verify that");
  tet_infoline ("this function reads the wide characters which are included");
  tet_infoline ("between `[' and ']' and this function stores the multibyte");
  tet_infoline ("character sequence which are made of the sequence of the");
  tet_infoline ("wide characters by wcsrtombs() with initial conversion");
  tet_infoline ("state. Especially check the null character is stood at the");
  tet_infoline ("end of the multibyte character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile16", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile16'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = fopen ("testfile16", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile16'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, conv, &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, (bytelen + 1)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test17()
{
  wchar_t wcs1[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'}, wcs2[BUFSIZ];
  wchar_t conv[] = {L'%', L'l', L'[', L'A', 0x00C4, 0x3042, L']', L'\0'};
  char *str = "AÄAああいあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test17");
  tet_infoline ("When a conversion wide character is `l[...]', verify that");
  tet_infoline ("this function reads the wide characters which are included");
  tet_infoline ("between `[' and ']' and this function stores them.");
  tet_infoline ("Especially check the null wide character is stood at the");
  tet_infoline ("end of the wide character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile17", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile17'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile17", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile17'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, conv, &wcs2);
  DBUG_PRINTF ("test17", "wcs1: %ls, wcs2: %ls", wcs1, wcs2);

  if (is_wcs_same (wcs1, wcs2, sizeof(wcs1) / sizeof(wcs1[0])))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test18()
{
  wchar_t wcs[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'};
  wchar_t conv[] = {L'%', L'[', L'^', L'B', 0x00FC, 0x3044, L']', L'\0'};
  wchar_t *p;
  const wchar_t **pp;
  char *str1 = "AÄAああいあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test18");
  tet_infoline ("When a conversion wide character is `[^...]', verify that");
  tet_infoline ("this function reads the wide characters which are not");
  tet_infoline ("included between `[^' and ']' and this function stores the");
  tet_infoline ("multibyte character sequence as same that are made of the");
  tet_infoline ("sequence of the wide characters by wcsrtombs() with initial");
  tet_infoline ("conversion state. Especially check the null character is");
  tet_infoline ("stood at the end of the multibyte character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile18", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile18'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = fopen ("testfile18", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile18'");
      DBUG_VOID_RETURN;
    }

  ret = va_fwscanf (fp, conv, &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);
  DBUG_PRINTF ("test18", "str2: %s, str3: %s", str2, str3);

  if (memcmp (str2, str3, (bytelen + 1)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test19()
{
  wchar_t wcs1[] = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'}, wcs2[BUFSIZ];
  wchar_t conv[] = {L'%', L'l', L'[', L'^', L'B', 0x00FC, 0x3044, L']', L'\0'};
  char *str = "AÄAああいあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test19");
  tet_infoline ("When a conversion wide character is `l[^...]', verify that");
  tet_infoline ("this function reads a sequence of the wide characters which");
  tet_infoline ("is not included between `[^' and ']', and stores them.");
  tet_infoline ("Especially check the null character is stood at the end of");
  tet_infoline ("the multibyte character string.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile19", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile19'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile19", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile19'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, conv, &wcs2);
  DBUG_PRINTF ("test19", "wcs1: %ls, wcs2: %ls", wcs1, wcs2);

  if (is_wcs_same (wcs1, wcs2, sizeof(wcs1) / sizeof(wcs1[0])))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test20()
{
  char a = 4;
  void *p1, *p2;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test20");
  tet_infoline ("When a conversion wide character is `p', verify that this");
  tet_infoline ("function stores the pointer value.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile20", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile20'");
      DBUG_VOID_RETURN;
    }
  p1 = &a;
  fprintf (fp, "%p", p1);
  fclose (fp);

  if ((fp = fopen ("testfile20", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile20'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%p", &p2);

  if (*(char *)p2 == 4)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character p is not interpreted correctly");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


#define TEST21_WCHAR_NUM 7
private void
test21()
{
  char *str = "AÄAああいあ";
  wchar_t wc1, wc2, wcs[BUFSIZ];
  FILE *fp;
  int  n, ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test21");
  tet_infoline ("When a conversion wide character is `n', verify that no");
  tet_infoline ("input is consumed and this function stores the number of");
  tet_infoline ("the wide character which the function has read.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile21", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile21'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("testfile21", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile21'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%lc%lcA%ls%n", &wc1, &wc2, wcs, &n);

  if (n == TEST21_WCHAR_NUM)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character n is not interpreted correctly");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test22()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test22");
  tet_infoline ("When a conversion wide character is `%' (i.e. the");
  tet_infoline ("orthography is `%%', verify that it matched a wide");
  tet_infoline ("character %.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile22", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile22'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%%9");
  fclose (fp);

  if ((fp = fopen ("testfile22", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile22'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%%%d", &n);

  if (n == 9)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character % is not interpreted correctly");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test23()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test23");
  tet_infoline ("When this function has not done any conversion by a read");
  tet_infoline ("error, verify that this function returns EOF.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile23", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile23'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", "");
  fclose (fp);

  if ((fp = fopen ("testfile23", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile23'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%d", &n);

  if (ret == EOF)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test23", "Return Value: %d", ret);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test24()
{
  FILE *fp;
  int  ret, n;
  char str[BUFSIZ];
  wchar_t wc;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test24");
  tet_infoline ("When this function has done one or more conversion, verify");
  tet_infoline ("that this function returns the number of the conversion.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile24", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile24'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "123abAあいう");
  fclose (fp);

  if ((fp = fopen ("testfile24", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile24'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%dab%lc%s", &n, &wc, str);

  if (ret == 3)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test24", "Return Value: %d", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test25()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test25");
  tet_infoline ("After this function execute correctly, verify that the");
  tet_infoline ("stream is changed to wide character oriented.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile25", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile25'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "9abc");
  fclose (fp);

  if ((fp = fopen ("testfile25", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile25'");
      DBUG_VOID_RETURN;
    }
  
  ret = va_fwscanf (fp, L"%d", &n);

  if (fwide (fp, 0) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
