/*
 * LI18NUX2K.L1/base/vswprintf/vswprintf.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <locale.h>
#include <wchar.h>

private char *test_locale = "LTP_1.UTF-8";

#if (BUFSIZ > 1024)
# define OUTPUT_WCS_MAX BUFSIZ
#else
# define OUTPUT_WCS_MAX 1024
#endif
private wchar_t output[OUTPUT_WCS_MAX];

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();
private void test16();
private void test17();
private void test18();
private void test19();
private void test20();
private void test21();
private void test22();
private void test23();
private void test24();
private void test25();
private void test26();
private void test27();
private void test28();
private void test29();
private void test30();
private void test31();
private void test32();
private void test33();
private void test34();
private void test35();
private void test36();
private void test37();
private void test38();
private void test39();
private void test40();
private void test41();
private void test42();
private void test43();
private void test44();
private void test45();
private void test46();
private void test47();
private void test48();
private void test49();
private void test50();
private void test51();
private void test52();

#define NO_TESTS 52
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,   1,
	test2,   2,
	test3,   3,
	test4,   4,
	test5,   5,
	test6,   6,
	test7,   7,
	test8,   8,
	test9,   9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	test25, 25,
	test26, 26,
	test27, 27,
	test28, 28,
	test29, 29,
	test30, 30,
	test31, 31,
	test32, 32,
	test33, 33,
	test34, 34,
	test35, 35,
	test36, 36,
	test37, 37,
	test38, 38,
	test39, 39,
	test40, 40,
	test41, 41,
	test42, 42,
	test43, 43,
	test44, 44,
	test45, 45,
	test46, 46,
	test47, 47,
	test48, 48,
	test49, 49,
	test50, 50,
	test51, 51,
	test52, 52,
	NULL, 0
};


/* Like swprintf with using vswprintf(). */
private int
va_swprintf (wchar_t *s, size_t n, wchar_t *format, ...)
{
  va_list argp;
  int ret;

  va_start (argp, format);
  ret = vswprintf (s, n, format, argp);
  va_end (argp);

  return (ret);
}


/* If field width is defined with decimal number, the length of output
   character is less than that decimal number, a `0' flag is not included
   in conversion specification and a `-' is not included in conversion
   specification, verify this function pads with spaces on the left. */
private void
test1()
{
  int  ret;
  char *str = "1234";
  wchar_t *ans = L"    1234";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If field width is defined with decimal number, the length of "
	       "output character is less than that decimal number, a `0' "
	       "flag is not included in conversion specification and a `-' "
	       "is not included in conversion specification, verify this "
	       "function pads with spaces on the left.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%8s", "1234");

  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Spaces are not padded with correctly.");
      DBUG_PRINTF ("test1", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If field width is defined with decimal number, the length of output
   character is less than that decimal number, and a `-' is included in
   conversion specification, verify this function pads with spaces on the
   right whether a `0' flag is included in conversion specification or not. */
private void
test2()
{
  int  ret;
  size_t anslen, len;
  char *str1 = "1234", *str2 = "1234";
  wchar_t *ans1 = L"1234    ", *ans2 = L"1234    ";
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If field width is defined with decimal number, the length of "
	       "output character is less than that decimal number, and a `-' "
	       "is included in conversion specification, verify this "
	       "function pads with spaces on the right whether a `0' flag is "
	       "included in conversion specification or not.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Without `0' flag. */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%-8s", str1);
  anslen = wcslen (ans1);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans1, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Spaces are not padded with correctly.");
      DBUG_PRINTF ("test2", "The output string: %ls", output);
    }

  /* With `0' flag. */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%-08s", str2);
  anslen = wcslen (ans2);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans2, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Spaces are not padded with correctly.");
      DBUG_PRINTF ("test2", "The output string: %ls", output);
    }
 
  PATH_XS_RPT (2);
  DBUG_VOID_RETURN;
}


/* If field width is defined with decimal number, the conversion specifier is
   `d', `i', `o', `u', `x', `X', `e', `E', `f', `g' or `G', the length of
   output character is less than that decimal number, a `0' flag is included
   in conversion specification, and a `-' is not included in conversion
   specification, verify this function padded with the wide character 0 on
   the left. */  
private void
test3()
{
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If field width is defined with decimal number, the conversion "
	       "specifier is `d', `i', `o', `u', `x', `X', `e', `E', `f', `g' "
	       "or `G', the length of output character is less than that "
	       "decimal number, a `0' flag is included in conversion "
	       "specification, and a `-' is not included in conversion "
	       "specification, verify this function padded with the wide "
	       "character 0 on the left.";
  wchar_t *iformat[] = {L"%06d", L"%06i", L"%06o", L"%06u",
		        L"%06x", L"%06X"};
  wchar_t *fformat[] = {L"%016e", L"%016E",
		        L"%016f",
		        L"%06g", L"%06G"};
  wchar_t *ians[] = {L"000001", L"000001", L"000001", L"000001", 
		     L"000001", L"000001"};
  wchar_t *fans[] = {L"00001*000000e+00", L"00001*000000E+00",
		     L"000000001*000000",
		     L"000001", L"000001"};
  int  i, inum, fnum;
  size_t anslen, len;

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  inum = sizeof(iformat) / sizeof(iformat[0]);
  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], 1);
      anslen = wcslen (ians[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ians[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Zeros are not padded correctly.");
	  DBUG_PRINTF ("test3", "The output string: %ls", output);
	}
    }

  fnum = sizeof(fformat) / sizeof(fformat[0]);
  for (i = 0; i< fnum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, fformat[i], 1.0);
      anslen = wcslen (fans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, fans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Zeros are not padded correctly.");
	  DBUG_PRINTF ("test3", "The output string: %s", output);
	}
    }

  PATH_XS_RPT (inum + fnum);
  DBUG_VOID_RETURN;
}


/* If field width is defined with decimal number and the length of output
   character is more than that decimal number, verify this function outputs
   the whole conversion results. */
private void
test4()
{
  int  ret;
  char *str = "12345678";
  wchar_t *ans = L"1234";
  size_t len, anslen;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If field width is defined with decimal number and the length "
	       "of output character is more than that decimal number, verify "
	       "this function outputs the whole conversion results.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%4s", str);

  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, 4) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Characters are not written correctly.");
      DBUG_PRINTF ("test4", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* Check this function ignores a `0' flag when the conversion specifier is
   `d', `i', `o', `u', `x' or `X' and conversion specification includes
   precision. */ 
private void
test5()
{
  int  ret;
  int  i, inum;
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Check this function ignores a `0' flag when the conversion "
	       "specifier is `d', `i', `o', `u', `x' or `X' and conversion "
	       "specification includes precision.";
  wchar_t *iformat[] = {L"%06.0d", L"%06.0i", L"%06.0o", L"%06.0u",
		        L"%06.0x", L"%06.0X"};
  wchar_t *ians[] = {L"     1", L"     1", L"     1", L"     1", 
		     L"     1", L"     1"};

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  inum = sizeof(iformat) / sizeof(iformat[0]);
  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], 1);

      anslen = wcslen (ians[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ians[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The flag `0' is not ignored.");
	  DBUG_PRINTF ("test5", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* Check the sign and base precedes the padded 0 when a `0' flag is given. */ 
private void
test6()
{
  int  ret;
  wchar_t ans1[] = L"-0001234", ans2[] = L"0x000001";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Check the sign and base precedes the padded 0 when a `0' "
	       "flag is given.";

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Sign. */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%08d", -1234);
  anslen = wcslen (ans1);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans1, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Sign is not appended correctly.");
      DBUG_PRINTF ("test6", "The output string: %ls", output);
    }

  /* Base. */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%0#8x", 1);
  anslen = wcslen (ans2);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans2, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Base is not appended correctly.");
      DBUG_PRINTF ("test6", "The output string: %ls", output);
    }

  PATH_XS_RPT (2);
  DBUG_VOID_RETURN;
}


/* If field width is given in *, verify an argument of type int supplies the
   field width. */  
private void
test7()
{
  int  ret;
  wchar_t ans[] = L"    1234";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If field width is given in *, verify an argument of type int "
	       "supplies the field width.";

  DBUG_ENTER ("test7");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%*s", 8, "1234");

  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("`*' is not interpreted correctry.");
      DBUG_PRINTF ("test7", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If precision is defined with decimal number and the conversion specifier
   is `d', `i', `o', `u', `x' or `X', verify the precision is interpreted as
   the minimum number of digits to appear. */
private void
test8()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%.6d", L"%.6i", L"%.6o", L"%.6u",
		        L"%.6x", L"%.6X"};
  wchar_t *ians[] = {L"000001", L"000001", L"000001", L"000001", 
		     L"000001", L"000001"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is defined with decimal number and the "
	       "conversion specifier is `d', `i', `o', `u', `x' or `X', "
	       "verify the precision is interpreted as the minimum number "
	       "of digits to appear.";

  DBUG_ENTER ("test8");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], 1);
      anslen = wcslen (ians[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ians[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The precision is not influenced correctrly.");
	  DBUG_PRINTF ("test8", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If precision is defined with decimal number and the conversion specifier is
   `e', `E' or `f', verify the precision is interpreted as the number of digits
   to appears after the radix character. */
private void
test9()
{
  int  ret;
  wchar_t *fformat[] = {L"%.3e", L"%.3E", L"%.3f"};
  wchar_t *fans[] = {L"1*000e+00", L"1*000E+00", L"1*000"};
  int  i, fnum;
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is defined with decimal number and the "
	       "conversion specifier is `e', `E' or `f', verify the precision "
	       "is interpreted as the number of digits to appears after the "
	       "radix character.";

  DBUG_ENTER ("test9");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  fnum = sizeof(fformat) / sizeof(fformat[0]);

  for (i = 0; i< fnum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, fformat[i], 1.0);
      anslen = wcslen (fans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, fans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The precision is not influenced correctrly.");
	  DBUG_PRINTF ("test9", "The output string: %ls", output);
	}

    }

  PATH_XS_RPT (fnum);
  DBUG_VOID_RETURN;
}


/* If precision is defined with decimal number and the conversion specifier is
   `g' or `G', verify the precision is interpreted as the maximum number of
   significant digits. */
private void
test10()
{
  int  ret;
  int  i, fnum;
  double  value[] = {1234567890.0, 1234567890.0};
  wchar_t *fformat[] = {L"%.3g", L"%.3G"};
  wchar_t *fans[] = {L"1*23e+09", L"1*23E+09"};
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is defined with decimal number and the "
	       "conversion specifier is `g' or `G', verify the precision "
	       "is interpreted as the maximum number of significant digits.";

  DBUG_ENTER ("test10");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  fnum = sizeof(fformat) / sizeof(fformat[0]);

  for (i = 0; i< fnum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, fformat[i], value[i]);
      anslen = wcslen (fans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (fans[i], output, len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The precision is not influenced correctrly.");
	  DBUG_PRINTF ("test10", "The output string: %ls", output);
	}

    }

  PATH_XS_RPT (fnum);
  DBUG_VOID_RETURN;
}


/* If precision is defined with decimal number and the conversion specifier
   is `s', verify the precision is interpreted as the maximum number of wide
   characters to be printed. */
private void
test11()
{
  int  ret;
  FILE *fp;
  char str[] = "aäあiïい";
  wchar_t wcs[] = {L'a', 0x00E4, 0x3042, L'i', 0x00EF, 0x3044, L'\0'}; 
  wchar_t ans[] = {L'a', 0x00E4, 0x3042, L'\0'};
  size_t len, anslen;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is defined with decimal number and the "
	       "conversion specifier is `s', verify the precision is "
	       "interpreted as the maximum number of wide characters to be "
	       "printed.";

  DBUG_ENTER ("test11");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  anslen = wcslen (ans);

  /* %s */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%.3s", str);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (output, ans, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The precision is not influenced correctrly when conversion "
	      "specifier is `%s'.");
      DBUG_PRINTF ("test11", "The output string: %ls", output);
    }

  /* %ls */
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%.3ls", wcs);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (output, ans, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The precision is not influenced correctrly when conversion "
	      "specifier is `%ls'.");
      DBUG_PRINTF ("test11", "The output string: %ls", output);
    }

  PATH_XS_RPT (2);
  DBUG_VOID_RETURN;
}


/* If precision is given in *, verify an argument of type int supplies the
   precision. */
private void
test12()
{
  int  ret;
  wchar_t *ans = L"0001";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is given in *, verify an argument of type int "
	       "supplies the precision.";

  DBUG_ENTER ("test12");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%.*d", 4, 1);
  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("`*' is not interpreted correctry.");
      DBUG_PRINTF ("test12", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If precision is formed only .(period) into, verify precision is treated
   as 0. */
private void
test13()
{
  int  ret;
  wchar_t *ans = L"1";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If precision is formed only .(period) into, verify precision "
	       "is treated as 0.";

  DBUG_ENTER ("test13");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%.f", 1.0);
  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The precsion is not interpreted correctry.");
      DBUG_PRINTF ("test13", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If a `+' flag is included in conversion specification, verify the result of
   a signed conversion begins with a sign(+ or -). */
private void
test14()
{
  int  ret;
  FILE *fp;
  wchar_t *ans = L"+1";
  size_t anslen, len;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `+' flag is included in conversion specification, verify "
	       "the result of a signed conversion begins with a sign(+ or -).";

  DBUG_ENTER ("test14");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%+d", 1);
  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (ans, output, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The `+' flag is not interpreted correctry.");
      DBUG_PRINTF ("test14", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If a `'(space) flag is included in conversion specification, verify the
   result of a signed conversion begins with a space when the first wide
   character of the conversion results is not a sign or a signed conversion
   results in no wide characters. */
private void
test15()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"% d", L"% d"};
  int  integ[] = {1, -1};
  wchar_t *ans[] = {L" 1", L"-1"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `'(space) flag is included in conversion specification, "
	       "verify the result of a signed conversion begins with a space "
	       "when the first wide character of the conversion results is "
	       "not a sign or a signed conversion results in no wide "
	       "characters.";

  DBUG_ENTER ("test15");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The precision is not influenced correctrly.");
	  DBUG_PRINTF ("test15", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a `#' is included in conversion specification and the conversion
   specifier is `o', verify the result of a conversion begins with a 0. */ 
private void
test16()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%#o"};
  unsigned int integ[] = {1};
  wchar_t *ans[] = {L"01"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `#' is included in conversion specification and the "
	       "conversion specifier is `o', verify the result of a "
	       "conversion begins with a 0.";

  DBUG_ENTER ("test16");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The `#' flag is not interpreted correctrly.");
	  DBUG_PRINTF ("test16", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a `#' is included in conversion specification and the conversion
   specifier is `x' or `X', verify the result of a conversion begins with a
   0x or 0X. */
private void
test17()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%#x", L"%#X"};
  unsigned int integ[] = {1, 1};
  wchar_t *ans[] = {L"0x1", L"0X1"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `#' is included in conversion specification and the "
	       "conversion specifier is `x' or `X', verify the result of a "
	       "conversion begins with a 0x or 0X.";

  DBUG_ENTER ("test17");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The `#' flag is not interpreted correctrly.");
	  DBUG_PRINTF ("test17", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a `#' is included in conversion specification and the conversion
   specifier is `e', `E', `f', `g' or `G', verify the result of a
   conversion includes the radix character. */
private void
test18()
{
  int  ret;
  size_t anslen, len;
  int  i, fnum;
  wchar_t *fformat[] = {L"%#.0e", L"%#.0E", L"%#.0f",
		        L"%#.0g", L"%#.0G",};
  double fraction[] = {1.0, 1.0, 1.0, 1.0, 1.0};
  wchar_t *ans[] = {L"1*e+00", L"1*E+00", L"1*", L"1*", L"1*"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `#' is included in conversion specification and the "
	       "conversion specifier is `e', `E', `f', `g' or `G', verify "
	       "the result of a conversion includes the radix character.";

  DBUG_ENTER ("test18");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  fnum = sizeof(fformat) / sizeof(fformat[0]);

  for (i = 0; i< fnum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, fformat[i], fraction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The `#' flag is not interpreted correctrly.");
	  DBUG_PRINTF ("test18", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (fnum);
  DBUG_VOID_RETURN;
}


/* If a `#' is included in conversion specification and the conversion
   specifier is `g' or `G', verify the trailing zeros after radix character
   is not removed from the results. */
private void
test19()
{
  int  ret;
  size_t anslen, len;
  int  i, fnum;
  wchar_t *fformat[] = {L"%#.2g", L"%#.2G",};
  double fraction[] = {1.0, 1.0};
  wchar_t *ans[] = {L"1*0", L"1*0"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a `#' is included in conversion specification and the "
	       "conversion specifier is `g' or `G', verify the trailing "
	       "zeros after radix character is not removed from the results.";

  DBUG_ENTER ("test19");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  fnum = sizeof(fformat) / sizeof(fformat[0]);

  for (i = 0; i< fnum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, fformat[i], fraction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The `#' flag is not interpreted correctrly.");
	  DBUG_PRINTF ("test19", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (fnum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `d' or `i', verify the int argument is
   converted to a signed decimal format. */
private void
test20()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%d", L"%i"};
  int integ[] = {1, -1};
  wchar_t *ans[] = {L"1", L"-1"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `d' or `i', verify the int "
	       "argument is converted to a signed decimal format.";

  DBUG_ENTER ("test20");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The specifier %%d or %%i is not interpreted correctrly.");
	  DBUG_PRINTF ("test20", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `d' or `i' and the precision is more than the
   number of digits, verify the conversion results is expanded with leading
   0. */
private void
test21()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%.3d", L"%.5i"};
  int integ[] = {1, 1};
  wchar_t *ans[] = {L"001", L"00001"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `d' or `i' and the precision is "
	       "more than the number of digits, verify the conversion results "
	       "is expanded with leading 0.";

  DBUG_ENTER ("test21");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The results is not expanded with leading 0 correctly.");
	  DBUG_PRINTF ("test21", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `u', verify the unsigned int argument is
   converted to a unsigned decimal format. */
private void
test22()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%u"};
  unsigned int integ[] = {1};
  wchar_t *ans[] = {L"1"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `u', verify the unsigned int "
	       "argument is converted to a unsigned decimal format.";

  DBUG_ENTER ("test22");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The specifier %%u is not interpreted correctrly.");
	  DBUG_PRINTF ("test22", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `u' and the precision is more than the number
   of digits, verify the conversion results is expanded with leading 0. */
private void
test23()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%.3u"};
  unsigned int integ[] = {1};
  wchar_t *ans[] = {L"001"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `u' and the precision is more "
	       "than the number of digits, verify the conversion results "
	       "is expanded with leading 0.";

  DBUG_ENTER ("test23");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The results is not expanded with leading 0 correctly.");
	  DBUG_PRINTF ("test23", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `o', verify the unsigned int argument is
   converted to a unsigned octal format. */
private void
test24()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%o"};
  unsigned int integ[] = {1};
  wchar_t *ans[] = {L"1"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `o', verify the unsigned int "
	       "argument is converted to a unsigned octal format.";

  DBUG_ENTER ("test24");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The specifier %%o is not interpreted correctrly.");
	  DBUG_PRINTF ("test24", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `o' and the precision is more than the number
   of digits, verify the conversion results is expanded with leading 0. */
private void
test25()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%.3u"};
  unsigned int integ[] = {1};
  wchar_t *ans[] = {L"001"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `o' and the precision is more "
	       "than the number of digits, verify the conversion results "
	       "is expanded with leading 0.";

  DBUG_ENTER ("test25");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The results is not expanded with leading 0 correctly.");
	  DBUG_PRINTF ("test25", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `x', verify the unsigned int argument is
   converted to a unsigned hexadecimal format (the letters `abcdef' are
   used). */
private void
test26()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%x"};
  unsigned int integ[] = {0xABC};
  wchar_t *ans[] = {L"abc"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `x', verify the unsigned int "
	       "argument is converted to a unsigned hexadecimal format (the "
	       "letters `abcdef' are used).";

  DBUG_ENTER ("test26");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The specifier %%x is not interpreted correctrly.");
	  DBUG_PRINTF ("test26", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `X', verify the unsigned int argument is
   converted to a unsigned hexadecimal format (the letters `ABCDEF' are
   used). */ 
private void
test27()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%X"};
  unsigned int integ[] = {0xef};
  wchar_t *ans[] = {L"EF"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `X', verify the unsigned int "
	       "argument is converted to a unsigned hexadecimal format (the "
	       "letters `ABCDEF' are used).";

  DBUG_ENTER ("test27");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The specifier %%X is not interpreted correctrly.");
	  DBUG_PRINTF ("test27", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `x' or `X', and the precision is more than
   the number of digits, verify the conversion results is expanded with
   leading 0. */
private void
test28()
{
  int  ret;
  size_t anslen, len;
  int  i, inum;
  wchar_t *iformat[] = {L"%.4x", L"%.5X"};
  unsigned int integ[] = {0x1A, 0XB2c};
  wchar_t *ans[] = {L"001a", L"00B2C"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `x' or `X', and the precision "
	       "is more than the number of digits, verify the conversion "
	       "results is expanded with leading 0.";

  DBUG_ENTER ("test28");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  inum = sizeof(iformat) / sizeof(iformat[0]);

  for (i = 0; i< inum; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, iformat[i], integ[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The results is not expanded with leading 0 correctly.");
	  DBUG_PRINTF ("test28", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (inum);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `f', verify the double argument is
   converted to decimal notation in the style [-]ddd.ddd. Especially check
   the radix character is according to the current locale. */
private void
test29()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%f", L"%f"};
  double flaction[] = {1.0, -1.0};
  wchar_t *ans[] = {L"1*000000", L"-1*000000"};
  char message[64] = "test locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `f', verify the double argument "
	       "is converted to decimal notation in the style [-]ddd.ddd. "
	       "Especially check the radix character is according to the "
	       "current locale.";

  DBUG_ENTER ("test29");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("the flag `f' is not interpreted correctrly.");
	  DBUG_PRINTF ("test29", "the output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `f' and the precision is not defined, verify
   the precision is taken as 6. */
private void
test30()
{
  int  ret, ret6;
  wchar_t output6[OUTPUT_WCS_MAX];
  FILE *fp;
  size_t len;
  wchar_t format[] = L"%f", format6[] = L"%.6f";
  double flaction = 1.0;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `f' and the precision is not "
	       "defined, verify the precision is taken as 6.";

  DBUG_ENTER ("test30");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret  = va_swprintf (output,  OUTPUT_WCS_MAX, format,  flaction);
  ret6 = va_swprintf (output6, OUTPUT_WCS_MAX, format6, flaction);

  len = (ret > ret6) ? ret : ret6;
  if (wmemcmp (output, output6, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The default of precision is not 6.");
      DBUG_PRINTF ("test30", "The output string(precision = 0): %ls", output);
      DBUG_PRINTF ("test30", "The output string(precision = 6): %ls", output6);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `f', the precision is 0 and a `#' flag is
   not included in conversion specification, verify no radix character
   appears. */
private void
test31()
{
  int  ret;
  wchar_t *format = L"%.0f";
  double flaction = 1.1234;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `f', the precision is 0 and a "
	       "`#' flag is not included in conversion specification, verify "
	       "no radix character appears.";

  DBUG_ENTER ("test31");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format, flaction);
  if (wcschr (output, L'*') == NULL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The radix character appears.");
      DBUG_PRINTF ("test31", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `e', verify the double argument is converted
   in the style [-]d.ddde+(or -)dd. Especially check the radix character is
   according to the current locale. */
private void
test32()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%e", L"%e"};
  double flaction[] = {1.0, -1.0};
  wchar_t *ans[] = {L"1*000000e+00", L"-1*000000e+00"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `e', verify the double argument "
	       "is converted in the style [-]d.ddde+(or -)dd. Especially "
	       "check the radix character is according to the current locale.";

  DBUG_ENTER ("test32");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The flag `e' is not interpreted correctrly.");
	  DBUG_PRINTF ("test32", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `E', verify the double argument is converted
   in the style [-]d.dddE+(or -)dd. Especially check the radix character is
   according to the current locale. */
private void
test33()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%E", L"%E"};
  double flaction[] = {1.000000E+00, -1.000000E+00};
  wchar_t *ans[] = {L"1*000000E+00", L"-1*000000E+00"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `E', verify the double argument "
	       "is converted in the style [-]d.dddE+(or -)dd. Especially "
	       "check the radix character is according to the current locale.";

  DBUG_ENTER ("test33");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The flag `E' is not interpreted correctrly.");
	  DBUG_PRINTF ("test33", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `e' or `E' and the precision is not
   defined, verify precision is taken as 6. */
private void
test34()
{
  int  ret, ret6;
  wchar_t output6[OUTPUT_WCS_MAX];
  size_t len;
  wchar_t formate[] = L"%e", formate6[] = L"%.6e";
  wchar_t formatE[] = L"%E", formatE6[] = L"%.6E";
  double flaction = 1.0;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `e' or `E' and the precision is "
	       "not defined, verify precision is taken as 6.";

  DBUG_ENTER ("test34");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret  = va_swprintf (output,  OUTPUT_WCS_MAX, formate,  flaction);
  ret6 = va_swprintf (output6, OUTPUT_WCS_MAX, formate6, flaction);
  len = (ret > ret6) ? ret : ret6;
  if (wmemcmp (output, output6, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("%%e: The default of precision is not 6.");
      DBUG_PRINTF ("test34", "The output string(precision = ): %ls", output);
      DBUG_PRINTF ("test34", "The output string(precision = 6): %ls", output6);
    }

  ret  = va_swprintf (output,  OUTPUT_WCS_MAX, formatE,  flaction);
  ret6 = va_swprintf (output6, OUTPUT_WCS_MAX, formatE6, flaction);
  len = (ret > ret6) ? ret : ret6;
  if (wmemcmp (output, output6, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("%%E: The default of precision is not 6.");
      DBUG_PRINTF ("test34", "The output string(precision = ): %ls", output);
      DBUG_PRINTF ("test34", "The output string(precision = 6): %ls", output6);
    }

  PATH_XS_RPT (2);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `e' or `E', the precision is 0 and a `#'
   flag is not included in conversion specification, verify no radix
   character appears. */
private void
test35()
{
  int  ret;
  wchar_t *formate = L"%.0e", *formatE = L"%.0E";
  double flaction = 1.1234;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `e' or `E', the precision is 0 "
	       "and a `#' flag is not included in conversion specification, "
	       "verify no radix character appears.";

  DBUG_ENTER ("test35");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, formate, flaction);
  if (wcschr (output, L'*') == NULL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("%%e: The radix character appears.");
      DBUG_PRINTF ("test35", "The output string: %ls", output);
    }

  ret = va_swprintf (output, OUTPUT_WCS_MAX, formatE, flaction);
  if (wcschr (output, L'*') == NULL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("%%E: The radix character appears.");
      DBUG_PRINTF ("test35", "The output string: %ls", output);
    }

  PATH_XS_RPT (2);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `g', if the exponent of the conversion
   results is less than -4 or greater than or equal to precision, verify the
   double argument is converted as same style as `e'. (But if precision is 0,
   it is taken as 1.) Especially check the radix character is according to
   the current locale. */
private void
test36()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%g", L"%.4g", L"%.0g"};
  double flaction[] = {0.00001, -10000, 20};
  wchar_t *ans[] = {L"1e-05", L"-1e+04", L"2e+01"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `g', if the exponent of the "
	       "conversion results is less than -4 or greater than or equal "
	       "to precision, verify the double argument is converted as same "
	       "style as `e'. (But if precision is 0, it is taken as 1.) "
	       "Especially check the radix character is according to the "
	       "current locale.";

  DBUG_ENTER ("test36");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The conversion specifier `g' is not interpreted correctly.");
	  DBUG_PRINTF ("test36", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `G', and if the exponent of the
   conversion results is less than -4 or greater than or equal to
   precision, verify the double argument is converted as same style as `E'.
   (But if precision is 0, it is taken as 1.) Especially check the radix
   character is according to the current locale. */
private void
test37()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%G", L"%.4G", L"%.0G"};
  double flaction[] = {0.00001, -10000, 20};
  wchar_t *ans[] = {L"1E-05", L"-1E+04", L"2E+01"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `G', and if the exponent of "
	       "the conversion results is less than -4 or greater than or "
	       "equal to precision, verify the double argument is converted "
	       "as same style as `E'. (But if precision is 0, it is taken as "
	       "1.) Especially check the radix character is according to the "
	       "current locale.";

  DBUG_ENTER ("test37");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The conversion specifier `G' is not interpreted correctly.");
	  DBUG_PRINTF ("test37", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `g' or `G', and if the exponent of the
   conversion results is greater than or equal to -4 and less than precision,
   verify the double argument is converted as same style as `f'. (But if
   precision is 0, it is taken as 1.) */
private void
test38()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%g", L"%.4g", L"%.0g", 
                       L"%G", L"%.4G", L"%.0G"};
  double flaction[] = {1.2345, -1234, 2,
                       1.2345, -1234, 2};
  wchar_t *ans[] = {L"1*2345", L"-1234", L"2",
                    L"1*2345", L"-1234", L"2"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `g' or `G', and if the "
	       "exponent of the conversion results is greater than or equal "
	       "to -4 and less than precision, verify the double argument is "
	       "converted as same style as `f'. (But if precision is 0, it "
	       "is taken as 1.)";

  DBUG_ENTER ("test38");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The conversion specifier `g' or `G' is not interpreted "
		  "correctrly.");
	  DBUG_PRINTF ("test38", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `g' or `G', verify the trailing 0 are removed
   from the fractional portion of the results. */
private void
test39()
{
  int  ret;
  size_t anslen, len;
  int  i, num;
  wchar_t *format[] = {L"%g", L"%G"};
  double flaction[] = {1.20000, 1.20000};
  wchar_t *ans[] = {L"1*2", L"1*2"};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `g' or `G', verify the trailing "
	       "0 are removed from the fractional portion of the results.";

  DBUG_ENTER ("test39");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      anslen = wcslen (ans[i]);
      len = (anslen > ret) ? anslen : ret;
      if (wmemcmp (output, ans[i], len) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The trailing 0 are not removed correctly.");
	  DBUG_PRINTF ("test39", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `g' or `G', verify a radix character is
   appears only when it follows digits. */
private void
test40()
{
  int  ret;
  int  i, num;
  wchar_t *format[] = {L"%g", L"%G", L"%g", L"%G"};
  double flaction[] = {1.20000, 1.20000, 1.00000, 1.00000};
  int ans[] = {1, 1, 0, 0};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `g' or `G', verify a radix "
	       "character is appears only when it follows digits.";

  DBUG_ENTER ("test40");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], flaction[i]);
      if ((wcschr (output, L'*') != NULL) == ans[i])
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The radix character does not appeared correctly.");
	  DBUG_PRINTF ("test40", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `c' and no `l' specifier is present, verify
   the int argument is converted to a wide character as if by calling the
   btowc() the resulting wide character is written. */
private void
test41()
{
  int  ret;
  int  i, num;
  wchar_t *format[] = {L"%c"};
  int  chara[] = {'a'};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `c' and no `l' specifier is "
	       "present, verify the int argument is converted to a wide "
	       "character as if by calling the btowc() the resulting wide "
	       "character is written.";

  DBUG_ENTER ("test41");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], chara[i]);
      if (output[0] == btowc(chara[i]))
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The conversion specifier `c' is not interpreted correctly.");
	  DBUG_PRINTF ("test41", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `c' and `l' specifier is present, verify
   the wint_t argument is written after converting wchar_t argument. */
private void
test42()
{
  int  ret;
  FILE *fp;
  int  i, num;
  wchar_t *format[] = {L"%lc", L"%lc", L"%lc"};
  wint_t  chara[] = {L'A', 0x00C4, 0x3042};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `c' and `l' specifier is "
	       "present, verify the wint_t argument is written after "
	       "converting wchar_t argument.";

  DBUG_ENTER ("test42");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  num = sizeof(format) / sizeof(format[0]);

  for (i = 0; i< num; i++)
    {
      ret = va_swprintf (output, OUTPUT_WCS_MAX, format[i], chara[i]);
      if (output[0] == chara[i])
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The conversion specifier `lc' is not interpreted "
		  "correctly.");
	  DBUG_PRINTF ("test42", "The output string: %ls", output);
	}
    }

  PATH_XS_RPT (num);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `s' and no `l' specifier is present, and
   if the precision is less than or equal to the length of the converted
   wide character, verify a multibyte character string starting at a
   pointer to character given by the argument is converted to a wide
   character string as if by calling the mbsrtowcs() function with initial
   conversion state and the wide characters, which are the top of string
   and which length is as same as the precision, are written. */
private void
test43()
{
  int  ret;
  wchar_t wcs[OUTPUT_WCS_MAX];
  size_t wcslen;
  int  precision = 4;
  wchar_t *format = L"%.*s";
  char *str = "AÄあAÄあ";
  mbstate_t state;
  char *ps;
  const char **pps;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `s' and no `l' specifier is "
	       "present, and if the precision is less than or equal to the "
	       "length of the converted wide character, verify a multibyte "
	       "character string starting at a pointer to character given by "
	       "the argument is converted to a wide character string as if "
	       "by calling the mbsrtowcs() function with initial conversion "
	       "state and the wide characters, which are the top of string "
	       "and which length is as same as the precision, are written.";

  DBUG_ENTER ("test43");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format, precision, str);
  ps  = str;
  pps = (const char **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  wcslen = mbsrtowcs (wcs, pps, strlen(str), &state);

  if (ret == precision && wmemcmp (output, wcs, precision) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The multibyte character is written incorrectly.");
      DBUG_PRINTF ("test43", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `s' and no `l' specifier is present, and
   if the precision is more than the length of the converted wide
   character, verify a multibyte character string starting at a pointer to
   character given by the argument is converted to a wide character string
   as if by calling the mbsrtowcs() function with initial conversion state
   and the wide character string is written. Especially check the null wide
   character is stood at the end of wide character string. */
private void
test44()
{
  int  ret;
  wchar_t wcs[OUTPUT_WCS_MAX];
  size_t len, wcslen;
  wchar_t *format = L"%.8s";
  char *str = "AÄあAÄあ";
  mbstate_t state;
  char *ps;
  const char **pps;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `s' and no `l' specifier is "
	       "present, and if the precision is more than the length of the "
	       "converted wide character, verify a multibyte character "
	       "string starting at a pointer to character given by the "
	       "argument is converted to a wide character string as if by "
	       "calling the mbsrtowcs() function with initial conversion "
	       "state and the wide character string is written. Especially "
	       "check the null wide character is stood at the end of wide "
	       "character string.";

  DBUG_ENTER ("test44");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format, str);
  ps  = str;
  pps = (const char **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  wcslen = mbsrtowcs (wcs, pps, (strlen(str) + 1), &state);

  len = ((wcslen > ret) ? wcslen : ret) + 1;
  if (wmemcmp (output, wcs, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The multibyte character is written incorrectly.");
      DBUG_PRINTF ("test44", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `s' and `l' specifier is present, and if
   the precision is less than or equal to the length of the converted wide
   character, verify the wide characters, which are the top of the string
   starting at a point to wide character given by the argument and which
   length is as same as the precision, are written. */
private void
test45()
{
  int  ret;
  size_t len, anslen;
  wchar_t *format = L"%.4ls";
  wchar_t str[] = {L'A', 0x00C4, 0x3042, L'A', 0x00C4, 0x3042, L'\0'};
  wchar_t ans[] = {L'A', 0x00C4, 0x3042, L'A', L'\0'};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `s' and `l' specifier is "
	       "present, and if the precision is less than or equal to the "
	       "length of the converted wide character, verify the wide "
	       "characters, which are the top of the string starting at a "
	       "point to wide character given by the argument and which "
	       "length is as same as the precision, are written.";

  DBUG_ENTER ("test45");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format, str);
  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (output, ans, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The multibyte character is written incorrectly.");
      DBUG_PRINTF ("test45", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* When a conversion specifier is `s' and `l' specifier is present, and the
   precision is more than the length of the converted wide character, verify
   a wide character string starting at a pointer to wide character given by
   the argument is written. Especially check the null wide character is stood
   at the end of wide character string. */
private void
test46()
{
  int  ret;
  FILE *fp;
  size_t len, anslen;
  wchar_t *format = L"%.8ls";
  wchar_t str[] = {L'A', 0x00C4, 0x3042, L'A', 0x00C4, 0x3042, L'\0'};
  wchar_t ans[] = {L'A', 0x00C4, 0x3042, L'A', 0x00C4, 0x3042, L'\0'};
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When a conversion specifier is `s' and `l' specifier is "
	       "present, and the precision is more than the length of the "
	       "converted wide character, verify a wide character string "
	       "starting at a pointer to wide character given by the "
	       "argument is written. Especially check the null wide "
	       "character is stood at the end of wide character string.";

  DBUG_ENTER ("test46");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format, str);
  anslen = wcslen (ans);
  len = ((anslen > ret) ? anslen : ret) + 1;
  if (wmemcmp (output, ans, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The multibyte character is written incorrectly.");
      DBUG_PRINTF ("test46", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `p', verify the argument which is a pointer to
   void is converted to a sequence of printable wide characters. */
private void
test47()
{
  char a = 4;
  void *p1, *p2;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `p', verify the argument which "
	       "is a pointer to void is converted to a sequence of printable "
	       "wide characters.";

  DBUG_ENTER ("test47");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  p1 = &a;
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%p", p1);
  swscanf (output, L"%p", &p2);

  if (p1 == p2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character p is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


#define TEST48_WCHAR_NUM 9
/* If a conversion specifier is `n', verify this function stores the number
   of the wide character, which the function has written, to the pointer to
   an integer which is given by the argument. */
private void
test48()
{
  char *str = "AÄAああいあ";
  int  a = 14;
  FILE *fp;
  int  n, ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `n', verify this function "
	       "stores the number of the wide character, which the function "
	       "has written, to the pointer to an integer which is given by "
	       "the argument.";

  DBUG_ENTER ("test48");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%s%d%n", str, a, &n);

  if (n == TEST48_WCHAR_NUM)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character n is not interpreted correctly");
      DBUG_PRINTF ("test48", "%%n has %d", n);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion specifier is `%', verify this function output a % wide
   character. */ 
private void
test49()
{
  int  ret;
  size_t len, anslen;
  wchar_t *format = L"%%";
  wchar_t *ans = L"%";
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specifier is `%', verify this function output "
	       "a % wide character.";

  DBUG_ENTER ("test49");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = va_swprintf (output, OUTPUT_WCS_MAX, format);
  anslen = wcslen (ans);
  len = (anslen > ret) ? anslen : ret;
  if (wmemcmp (output, ans, len) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The conversion specifier `%' is not interpreted correctly.");
      DBUG_PRINTF ("test49", "The output string: %ls", output);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* If the total length of the wide character which is made by conversion is
   more than n (including the terminating null wide character), verify this
   function returns negative value and outputs only n wide character. */
private void
test50()
{
  int  ret;
  size_t  len = 5;
  wchar_t wcs[] = L"1234567890";
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the total length of the wide character which is made by "
	       "conversion is more than n (including the terminating null "
	       "wide character), verify this function returns negative value "
	       "and outputs only n wide character.";

  DBUG_ENTER ("test50");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (output, L'\0', OUTPUT_WCS_MAX);
  ret = va_swprintf (output, len, wcs);

  DBUG_PRINTF ("test50", "Return Value: %d", ret);
  DBUG_PRINTF ("test50", "len = %d, output_len = %d", len, wcslen(output));
  DBUG_PRINTF ("test50", "Output Characters: %ls", output);

  if (ret == -1)
    if (wcslen (output) <= len)
      PATH_TRACE;
    else
      {
	in_rpt ("The number of written wide character is not correct.");
	DBUG_PRINTF ("test50", "The number: %s", wcslen (output));
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test50", "Return Value: %d", ret);
    }

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


#define TEST51_WCHAR_NUM 9
/* If no output error is occurred, verify this function returns the number
   of the written wide characters (excluding the terminating null wide
   character). */
private void
test51()
{
  char *str = "AÄAああいあ";
  int  a = 14;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If no output error is occurred, verify this function returns "
	       "the number of the written wide characters (excluding the "
	       "terminating null wide character).";

  DBUG_ENTER ("test51");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%s%d", str, a);

  if (ret == TEST51_WCHAR_NUM)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test51", "The return value is %d", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


#define TEST52_CHAR_NUM 509
/* Verify the limits of conversion character is at least 509. */
private void
test52()
{
  int  ret;
  char str[TEST52_CHAR_NUM + 1];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Verify the limits of conversion character is at least 509.";

  DBUG_ENTER ("test52");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  memset (str, 'A', TEST52_CHAR_NUM);
  str[TEST52_CHAR_NUM] = '\0';
  
  ret = va_swprintf (output, OUTPUT_WCS_MAX, L"%s", str);

  if (ret >= TEST52_CHAR_NUM)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The limits of conversion is not reached 509");
      DBUG_PRINTF ("test52", "The return value is %d", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
