/*
 * LI18NUX2K.L1/base/wcscasecmp/wcscasecmp.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#define __USE_GNU
#include <wchar.h>

private char *test_locale = "LTP_1.UTF-8";

private wchar_t	wcs[] = {0xFF43, 0xFF43, 0xFF43, L'\0'};
private wchar_t	les[] = {0xFF21, 0xFF21, 0xFF21, L'\0'};
private wchar_t	equ[] = {0xFF23, 0xFF23, 0xFF23, L'\0'};
private wchar_t	grt[] = {0xFF25, 0xFF25, 0xFF25, L'\0'};

private void test1();
private void test2();
private void test3();

#define NO_TESTS 3
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
      {test1, 1},
      {test2, 2},
      {test3, 3},
      {NULL, 0}
};


private void
test1()
{
  int  rval;
  int  pathok = 0;

  DBUG_ENTER ("test1");
  tet_infoline ("When a wide character string starting at s1 is less than a");
  tet_infoline ("wide character string starting at s2 except for case");
  tet_infoline ("distinctions, verify this function returns negative value.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;
  rval = wcscasecmp (les, wcs);

  if (rval < 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
                
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test2()
{
  int  rval;
  int  pathok = 0;

  DBUG_ENTER ("test2");
  tet_infoline ("If a wide character string starting at s1 and a wide");
  tet_infoline ("character string starting at s2 are equal except for case");
  tet_infoline ("distinctions, verify this function returns 0.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;
  rval = wcscasecmp (equ, wcs);

  if (rval == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
                
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test3()
{
  int  rval;
  int  pathok = 0;

  DBUG_ENTER ("test3");
  tet_infoline ("If a wide character string starting at s1 is greater than a");
  tet_infoline ("wide character string starting at s2 except for case");
  tet_infoline ("distinctions, verify this function returns positive value.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;
  rval = wcscasecmp (grt, wcs);

  if (rval > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
                
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
