/*
 * LI18NUX2K.L1/base/wcsnrtombs/wcsnrtombs.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <errno.h>

#define __USE_GNU
#include <wchar.h>

#define STRING_MAX 16
private char *test_locale = "LTP_1.UTF-8";

/* This test checks 3 types of string.
   1st - LATIN SMALL LETTER A and B
   2nd - LATIN SMALL LETTER A WITH DIAERESIS AND B
   3rd - HIRAGANA LETTER A and LATIN SMALL LETTER B */
#define TYPE_NUM     3

private unsigned char correct_str[TYPE_NUM][8] = {
      {'A',  'B',  '\0'},
      {0xC3, 0xA4, 'B',  '\0'},
      {0xE3, 0x81, 0x82, 'B', '\0'}
};

private wchar_t correct_wstr[TYPE_NUM][4] = {
      {L'A',   L'B', L'\0'},
      {0x00E4, L'B', L'\0'},
      {0x3042, L'B', L'\0'}
};

private size_t str_len[TYPE_NUM] = {
    2,
    3,
    4
};
      
private size_t wstr_len[TYPE_NUM] = {
    2,
    2,
    2
};

private size_t first_char_bytes[TYPE_NUM] = {
    1,
    2,
    3
};

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();

#define NO_TESTS 7
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
      {test1, 1},
      {test2, 2},
      {test3, 3},
      {test4, 4},
      {test5, 5},
      {test6, 6},
      {test7, 7},
      {NULL, 0}
};

/*
 * test1
 */
private int
test1_core (int num)
{
  char dest[STRING_MAX];
  wchar_t *pw;
  const wchar_t **ppw;
  size_t  ret; 
  mbstate_t state;

  DBUG_ENTER ("test1_core");

  pw  = correct_wstr[num];
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (dest, ppw, wstr_len[num], str_len[num], &state);
  if (ret == str_len[num])
    if (memcmp (dest, correct_str[num], str_len[num]) == 0)
      if (pw != NULL && *pw == '\0')
	DBUG_RETURN (0);
      else
	in_rpt ("The address which src shows is not correct.");
    else
      in_rpt ("The character array stored at dest is not correct");
  else
    in_rpt ("The return value is not correct.");

  DBUG_RETURN (1);
}

private void
test1()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0, num;

  DBUG_ENTER ("test1");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" * dest is not NULL pointer.");
  tet_infoline (" * The terminating L`\0' locates at the less than or equal");
  tet_infoline ("   to nwc-th from the pointer which src stored.");
  tet_infoline (" * The wide character sequence starting at src is able to be");
  tet_infoline ("   converted to the multibyte character string.");
  tet_infoline ("* The byte length of converted multi byte characters");
  tet_infoline ("  (not including the terminating null character) is less");
  tet_infoline ("  than or equal to len.");
  tet_infoline (", verify the following:");
  tet_infoline ("* The src value is NULL.");
  tet_infoline ("* The return value is the byte length of the converted");
  tet_infoline ("  multibyte characters (not including the terminating null");
  tet_infoline ("  character).");
  tet_infoline ("* The converted multibyte character string is stored from");
  tet_infoline ("  dest.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  for (num = 0; num < TYPE_NUM; num++)
    {
      if (test1_core (num) == 0)
        PATH_TRACE;
      else
	xx_rpt (FAILURE);
    };

  PATH_XS_RPT(TYPE_NUM);
  DBUG_VOID_RETURN;
}


/*
 * test2
 */
private int
test2_core (int num)
{
  char dest[STRING_MAX];
  wchar_t *pw;
  const wchar_t **ppw;
  size_t  ret; 
  mbstate_t state;

  size_t conv_len;

  DBUG_ENTER ("test2_core");

  conv_len = first_char_bytes[num];
  memset (dest, '\0', STRING_MAX);

  pw  = correct_wstr[num];
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (dest, ppw, wstr_len[num], conv_len, &state);

  if (*pw == L'B')
    if (ret == conv_len)
      if (dest[conv_len] == L'\0')
	if (memcmp (dest, correct_str[num], conv_len) == 0)
	  DBUG_RETURN (0);
	else
	  in_rpt ("The converted characters are not stored from dest.");
      else
	in_rpt ("The byte length of converted multibyte character is not len.");
    else
      in_rpt ("The return value is not equal to len.");
  else
    in_rpt ("src does not point the next wide character sequence.");

  DBUG_RETURN (1);
}

private void
test2()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0, num;

  DBUG_ENTER ("test2");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" * dest is not NULL pointer.");
  tet_infoline (" * The terminating L`\0' locates at the less than or equal");
  tet_infoline ("   to nwc-th from the pointer which src stored.");
  tet_infoline (" * The wide character sequence starting at src is able to");
  tet_infoline ("   be converted to the multibyte character string.");
  tet_infoline (" * The byte length of converted multi byte characters (not");
  tet_infoline ("   including the terminating null character) is bigger than");
  tet_infoline ("   len. ");
  tet_infoline (", verify the following:");
  tet_infoline (" * The src is left pointing to the next wide character");
  tet_infoline ("   sequence to be converted.");
  tet_infoline (" * The return value is equal to len.");
  tet_infoline (" * The number of converted multibyte character (not");
  tet_infoline ("   including the terminating null character) is len.");
  tet_infoline (" * The converted multibyte character string is stored");
  tet_infoline ("   from dest.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  for (num = 0; num < TYPE_NUM; num++)
    {
      if (test2_core (num) == 0)
        PATH_TRACE;
      else
	xx_rpt (FAILURE);
    };

  PATH_XS_RPT(TYPE_NUM);
  DBUG_VOID_RETURN;
}

/*
 * test3
 */
private void
test3()
{
  char	message[64] = "Test Locale is ";
  int	pathok = 0;

  wchar_t wcs[] = {
      0x3042,		/* HIRAGANA LETTER A */
      0x00E4,           /* LATIN SMALL LETTER A WITH DIAERESIS */
      L'\0'
  };

  char	    dest[STRING_MAX];
  wchar_t   *pw;
  const wchar_t	**ppw;
  size_t    nms = 1;
  size_t    len = STRING_MAX;
  mbstate_t state;
  size_t    ret;

  wchar_t   *exp_pw = wcs + nms;  /* Skip HIRAGANA LETTER A */
  size_t    exp_ret = 3;          /* i.e. HIRAGANA LETTER A - 3 byte char */
  char	    exp_str[] = {
      0xE3, 0x81, 0x82,         /* HIRAGANA LETTER A */
      '\0'
  };

  DBUG_ENTER ("test3");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" * dest is not NULL pointer.");
  tet_infoline (" * The terminating L`\0' locates at the bigger than nwc-th");
  tet_infoline ("   from the pointer which src stored.");
  tet_infoline (" * The wide character sequence starting at src is able to be");
  tet_infoline ("   converted to the multibyte character string.");
  tet_infoline (" * The byte length of converted multi byte characters (not");
  tet_infoline ("   including the terminating null character) is less than");
  tet_infoline ("   or equal to len.");
  tet_infoline (", verify the following:");
  tet_infoline (" * The src is left pointing to the next wide character");
  tet_infoline ("   sequence to be converted.");
  tet_infoline (" * The return value is the byte length of the converted");
  tet_infoline ("   multibyte characters (not including the terminating null");
  tet_infoline ("   character).");
  tet_infoline (" * The converted multibyte character string is stored from");
  tet_infoline ("   dest.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  pw  = wcs;
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (dest, ppw, nms, len, &state);

  if (pw == exp_pw)
    if (ret == exp_ret)
      if (memcmp (dest, exp_str, ret) == 0)
	{
	  PATH_TRACE;
	}
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The multibyte character array stored at dest is incorrect");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The return value is not correct.");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The address which pw shows is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

/*
 * test4
 */
private void
test4()
{
  char	message[64] = "Test Locale is ";
  int	pathok = 0;

  wchar_t wcs[] = {
      0x00E4,           /* LATIN SMALL LETTER A WITH DIAERESIS */
      0x0001,		/* An invlid wide character will be in here. */
      L'A',		/* LATIN SMALL LETTER A */
      L'\0'
  };

  char	    dest[STRING_MAX];
  wchar_t   *pw;
  const wchar_t	**ppw;
  size_t    nms = 4;
  size_t    len = STRING_MAX;
  mbstate_t state;
  size_t    ret;

  wchar_t   *exp_pw = wcs + 1;	  /* Skip SMALL LETTER A WITH DIAERESIS */
  size_t    exp_ret = (size_t) -1;
  int       exp_errno = EILSEQ;


  DBUG_ENTER ("test4");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" * dest is not NULL pointer.");
  tet_infoline (" * An invalid wide character which cannot be converted to a");
  tet_infoline ("   multibyte character exists in the wide character");
  tet_infoline ("   sequence for conversion. ");
  tet_infoline (", verify the following:");
  tet_infoline (" * The src is left pointing to the the invalid wide");
  tet_infoline ("   character.");
  tet_infoline (" * The return value is (size_t)(-1).");
  tet_infoline (" * The global variable errno is set to EILSEQ.");

  /* Make wcs[1] invalid. */
  wcs[1] = wcs[1] << (sizeof (wchar_t) * 8 - 1);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  pw  = wcs;
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (dest, ppw, nms, len, &state);

  if (pw == exp_pw)
    if (ret == exp_ret)
      if (errno == exp_errno)
	{
	  PATH_TRACE;
	}
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The errno is not EILSEQ.");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The return value is not correct.");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The address which pw shows is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/*
 * test5
 */
private int
test5_core (int num)
{
  wchar_t *pw;
  const wchar_t **ppw;
  size_t  ret; 
  mbstate_t state;

  DBUG_ENTER ("test5_core");

  pw  = correct_wstr[num];
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (NULL, ppw, wstr_len[num], str_len[num], &state);
  if (ret == str_len[num])
    DBUG_RETURN (0);
  else
    in_rpt ("The return value is not correct.");

  DBUG_RETURN (1);
}

private void
test5()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0, num;

  DBUG_ENTER ("test5");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" dest is NULL pointer.");
  tet_infoline (" * The terminating L`\0' locates at the less than or equal");
  tet_infoline ("   to nwc-th from the pointer which src stored.");
  tet_infoline (" * The wide character sequence starting at src is able to");
  tet_infoline ("   be converted to the multibyte character string.");
  tet_infoline (" * The byte length of converted multi byte characters (not");
  tet_infoline ("   including the terminating null character) is less than");
  tet_infoline ("   or equal to len.");
  tet_infoline (" , verify the following:");
  tet_infoline (" * The return value is the byte length of the converted");
  tet_infoline ("   multibyte characters (not including the terminating null");
  tet_infoline ("   character).");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  for (num = 0; num < TYPE_NUM; num++)
    {
      if (test5_core (num) == 0)
        PATH_TRACE;
      else
	xx_rpt (FAILURE);
    };

  PATH_XS_RPT(TYPE_NUM);
  DBUG_VOID_RETURN;
}


/*
 * test6
 */
private int
test6_core (int num)
{
  wchar_t *pw;
  const wchar_t **ppw;
  size_t  ret; 
  mbstate_t state;

  size_t conv_len;

  DBUG_ENTER ("test6_core");

  conv_len = first_char_bytes[num];

  pw  = correct_wstr[num];
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (NULL, ppw, wstr_len[num], conv_len, &state);

  if (ret == str_len[num])
    DBUG_RETURN (0);
  else
    {
      char msg[256];
      sprintf (msg, "ret: %d, conv_len: %d\n", ret, conv_len);
      tet_infoline (msg);

      tet_infoline ("The return value is not equal to len.");
      tet_infoline ("mbsnrtowcs may not ignore the len.");
    }

  DBUG_RETURN (1);
}

private void
test6()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0, num;

  DBUG_ENTER ("test6");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" *  dest is NULL pointer.");
  tet_infoline (" * The terminating L`\0' locates at the less than or equal");
  tet_infoline ("   to nwc-th from the pointer which src stored.");
  tet_infoline (" * The wide character sequence starting at src is able to");
  tet_infoline ("   be converted to the multibyte character string.");
  tet_infoline (" * The byte length of converted multi byte characters (not");
  tet_infoline ("   including the terminating null character) is bigger than");
  tet_infoline ("   len.");
  tet_infoline (", verify the following:");
  tet_infoline (" * len is ignored.");
  tet_infoline (" * The return value is the byte length of the converted");
  tet_infoline ("   multibyte characters (not including the terminating null");
  tet_infoline ("   character).");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  for (num = 0; num < TYPE_NUM; num++)
    {
      if (test6_core (num) == 0)
        PATH_TRACE;
      else
	xx_rpt (FAILURE);
    };

  PATH_XS_RPT(TYPE_NUM);
  DBUG_VOID_RETURN;
}


/*
 * test7
 */
private void
test7()
{
  char	message[64] = "Test Locale is ";
  int	pathok = 0;

  wchar_t wcs[] = {
      0x00E4,           /* LATIN SMALL LETTER A WITH DIAERESIS */
      0x0001,		/* An invlid wide character will be in here. */
      L'A',		/* LATIN SMALL LETTER A */
      L'\0'
  };

  wchar_t   *pw;
  const wchar_t	**ppw;
  size_t    nms = 4;
  size_t    len = STRING_MAX;
  mbstate_t state;
  size_t    ret;

  size_t    exp_ret = (size_t) -1;
  int       exp_errno = EILSEQ;


  DBUG_ENTER ("test7");
  tet_infoline ("When the situation is the following:");
  tet_infoline (" * dest is NULL pointer.");
  tet_infoline (" * An invalid wide character which cannot be converted to a");
  tet_infoline ("   multibyte character exists in the wide character");
  tet_infoline ("   sequence for conversion. ");
  tet_infoline (", verify the following:");
  tet_infoline (" * The return value is (size_t)(-1).");
  tet_infoline (" * The global variable errno is set to EILSEQ.");

  /* Make wcs[1] invalid. */
  wcs[1] = wcs[1] << (sizeof (wchar_t) * 8 - 1);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  pw  = wcs;
  ppw = (const wchar_t **)&pw;
  memset (&state, '\0', sizeof(mbstate_t));

  ret = wcsnrtombs (NULL, ppw, nms, len, &state);

  if (ret == exp_ret)
    if (errno == exp_errno)
      {
	PATH_TRACE;
      }
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The errno is not EILSEQ.");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
