/*
 * LI18NUX2K.L1/base/wcsrtombs/wcsrtombs.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wchar.h>
#include <errno.h>

#define MAX_BYTE_LEN 32

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();

/* LATIN SMALL LETTER A and B */
private unsigned char correct_str1[]  = "AB";
private wchar_t       correct_wstr1[] = L"AB";
private size_t        correct_ret1    = 2;

/* LATIN SMALL LETTER A WITH DIAERESIS AND B */
private unsigned char correct_str2[]  = {0xC3, 0xA4, 'B', '\0'};
private wchar_t       correct_wstr2[] = {0x00E4, L'B', L'\0'};
private size_t        correct_ret2    = 3;

/* HIRAGANA LETTER A and LATIN SMALL LETTER B */
private unsigned char correct_str3[]  = {0xE3, 0x81, 0x82, 'B', '\0'};
private wchar_t       correct_wstr3[] = {0x3042, L'B', L'\0'};
private size_t        correct_ret3    = 4;

/* LATIN SMALL LETTER A and B, and null character, and A, the last A is not
     translated. */
private unsigned char correct_str4[]  = "AB";
private wchar_t       correct_wstr4[] = {L'A', L'B', L'\0', L'A'};;
private size_t        length_str4     = 4;
private size_t        correct_ret4    = 2;

/* LATIN SMALL LETTER A and incorrect wide character */
private unsigned char cor_chars[]	       = "A";
private wchar_t       incorrect_wstr[] = {L'A', 0x80000000, L'\0'};
private size_t        length_inc_str   = 5;
private size_t	      length_cor_chars  = 1;

#define NO_TESTS 3
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	NULL, 0
};

/* If src has no wide character which cannot be translated to multibyte
   character when the length of translated multibyte character is less than
   or equal to n, and null wide character is not in that range, verify this
   function returns the byte length of translated multibyte character
   sequence, and stores byte sequence which shows as same characters as the
   wide characters at dst, and stores the address which points the next
   wide character of the last converted wide character at src. */
private void
test1()
{
  size_t ret;
  wchar_t *ps;
  const wchar_t **pps;
  mbstate_t state;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If src has no wide character which cannot be translated to "
	       "multibyte character when the length of translated multibyte "
	       "character is less than or equal to n, and null wide "
	       "character is not in that range, verify this function returns "
	       "the byte length of translated multibyte character sequence, "
	       "and stores byte sequence which shows as same characters as "
	       "the wide characters at dst, and stores the address which "
	       "points the next wide character of the last converted wide "
	       "character at src.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale (LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ps  = correct_wstr1;
  pps = (const wchar_t **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  if ((ret = wcsrtombs (mbs, pps, correct_ret1, &state)) == correct_ret1)
    if (!memcmp(mbs, correct_str1, correct_ret1))
      if (ps != NULL && *ps == L'\0')
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The address which src shows is not correct.");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at dst is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
  
  ps  = correct_wstr2;
  pps = (const wchar_t **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  if ((ret = wcsrtombs (mbs, pps, correct_ret2, &state)) == correct_ret2)
    if (!memcmp(mbs, correct_str2, correct_ret2))
      if (ps != NULL && *ps == L'\0')
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The address which src shows is not correct.");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at dst is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct. ");
    }

  ps  = correct_wstr3;
  pps = (const wchar_t **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  if ((ret = wcsrtombs (mbs, pps, correct_ret3, &state)) == correct_ret3)
    if (!memcmp(mbs, correct_str3, correct_ret3))
      if (ps != NULL && *ps == L'\0')
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The address which src shows is not correct.");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at dst is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}


/* If src has no wide character which cannot be translated to multibyte
   character, and there is a null wide character in the wide characters,
   verify this function translates wide character to multibyte character
   from first to the null wide character and returns the byte length of the
   translated multibyte characters, and stores byte sequence which shows as
   same characters as the wide characters at dst, and stores the NULL at
   src. */
private void
test2()
{
  size_t ret;
  wchar_t *ps;
  const wchar_t **pps;
  mbstate_t state;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If src has no wide character which cannot be translated to "
	       "multibyte character, and there is a null wide character in "
	       "the wide characters, verify this function translates wide "
	       "character to multibyte character from first to the null wide "
	       "character and returns the byte length of the translated "
	       "multibyte characters, and stores byte sequence which shows "
	       "as same characters as the wide characters at dst, and stores "
	       "the NULL at src.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;
  
  strcat (message, test_locale);
  tet_infoline (message);

  ps  = correct_wstr4;
  pps = (const wchar_t **)&ps;
  memset (&state, '\0', sizeof(mbstate_t));
  if ((ret = wcsrtombs (mbs, pps, length_str4, &state)) == correct_ret4)
    if (!memcmp(mbs, correct_str4, correct_ret4))
      if (ps == NULL)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The address which src shows is not correct.");
	}
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at dst is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
 
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If src has a wide character which cannot be translated to multibyte
   character in the translation range, verify this function returns -1, and
   stores byte sequence which show as same characters as the wide
   characters from first to the last convertible character at dst and
   stores the address which point the next wide character of the last
   converted character at src, and set errno EILSEQ. */
private void
test3()
{
  size_t ret;
  wchar_t *ps;
  const wchar_t **pps;
  mbstate_t state;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If src has a wide character which cannot be translated to "
	       "multibyte character in the translation range, verify this "
	       "function returns -1, and stores byte sequence which show as "
	       "same characters as the wide characters from first to the "
	       "last convertible character at dst and stores the address "
	       "which point the next wide character of the last converted "
	       "character at src, and set errno EILSEQ.";
  
  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ps  = incorrect_wstr;
  pps = (const wchar_t **)&ps;
  errno = 0;
  memset (&state, '\0', sizeof(mbstate_t));
  if ((ret = wcsrtombs (mbs, pps, length_inc_str, &state)) == -1)
    if (!memcmp(mbs, cor_chars, length_cor_chars))
      if (ps == incorrect_wstr + length_cor_chars)
	if (errno == EILSEQ)
	  PATH_TRACE;
	else
	  {
	    PATH_TRACE;
	    xx_rpt (FAILURE);
	    in_rpt ("The errno is incorrect.");
	  }
      else
	{
	  PATH_TRACE;
	  xx_rpt (FAILURE);
	  in_rpt ("The terminated position of a wide character string is incorrect.");
	}
    else
      {
	PATH_TRACE;
	xx_rpt (FAILURE);
	in_rpt ("The terminated position of a multibyte character string is incorrect.");

      }
  else
    {
      PATH_TRACE;
      xx_rpt (FAILURE);
      in_rpt ("The return value is incorrect.");

    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
