/*
 * LI18NUX2K.L1/base/wcstombs/wcstombs.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wchar.h>

#define MAX_BYTE_LEN 32

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();

/* LATIN SMALL LETTER A and B */
private unsigned char correct_str1[]  = "AB";
private wchar_t       correct_wstr1[] = L"AB";
private size_t        correct_ret1    = 2;

/* LATIN SMALL LETTER A WITH DIAERESIS AND B */
private unsigned char correct_str2[]  = {0xC3, 0xA4, 'B', '\0'};
private wchar_t       correct_wstr2[] = {0x00E4, L'B', L'\0'};
private size_t        correct_ret2    = 3;

/* HIRAGANA LETTER A and LATIN SMALL LETTER B */
private unsigned char correct_str3[]  = {0xE3, 0x81, 0x82, 'B', '\0'};
private wchar_t       correct_wstr3[] = {0x3042, L'B', L'\0'};
private size_t        correct_ret3    = 4;

/* LATIN SMALL LETTER A and B, and null character, and A, the last A is not
     translated. */
private unsigned char correct_str4[]  = "AB";
private wchar_t       correct_wstr4[] = {L'A', L'B', L'\0', L'A'};;
private size_t        length_str4     = 4;
private size_t        correct_ret4    = 2;

/* LATIN SMALL LETTER A and incorrect wide character */
private wchar_t      incorrect_wstr[] = {L'A', 0x80000000, L'\0'};
private size_t       length_inc_str   = 5;

#define NO_TESTS 3
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	NULL, 0
};

/* If the wide character sequence starting at pwcs has no wide character
   which cannot be translated to multibyte character where the length of
   translated multibyte character is less than or equal to n, and null wide
   character is not in that range, verify this function returns the length
   of the byte sequence which shows as same characters as the wide
   characters and stores the byte sequence at the s. */
private void
test1()
{
  size_t   ret;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character sequence starting at pwcs has no wide "
	       "character which cannot be translated to multibyte character "
	       "where the length of translated multibyte character is less "
	       "than or equal to n, and null wide character is not in that "
	       "range, verify this function returns the length of the byte "
	       "sequence which shows as same characters as the wide "
	       "characters and stores the byte sequence at the s.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = wcstombs (mbs, correct_wstr1, correct_ret1)) == correct_ret1)
    if (!memcmp(mbs, correct_str1, correct_ret1))
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at s is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
  
  if ((ret = wcstombs (mbs, correct_wstr2, correct_ret2)) == correct_ret2)
    if (!memcmp(mbs, correct_str2, correct_ret2))
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at s is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct. [2]");
    }

  if ((ret = wcstombs (mbs, correct_wstr3, correct_ret3)) == correct_ret3)
    if (!memcmp(mbs, correct_str3, correct_ret3))
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at s is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}


/* If the wide character sequence starting at pwcs has no wide character which
   cannot be translated to multibyte character where the length of translated
   multibyte character is less than or equal to n, and null wide character is
   in that range, verify this function translates wide characters to multibyte
   characters from the first to the null wide character and returns the length
   of the byte sequence which shows as same characters as the wide characters
   and stores the byte sequence at s. */
private void
test2()
{
  size_t ret;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character sequence starting at pwcs has no wide "
	       "character which cannot be translated to multibyte character "
	       "where the length of translated multibyte character is less "
	       "than or equal to n, and null wide character is in that range, "
	       "verify this function translates wide characters to multibyte "
	       "characters from the first to the null wide character and "
	       "returns the length of the byte sequence which shows as same "
	       "characters as the wide characters and stores the byte sequence "
	       "at s.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  globok = 1;
  
  strcat (message, test_locale);
  tet_infoline (message);

  if ((ret = wcstombs (mbs, correct_wstr4, length_str4)) == correct_ret4)
    if (!memcmp(mbs, correct_str4, correct_ret4))
      PATH_TRACE;
    else
      {
	xx_rpt (FAILURE);
	in_rpt ("The multibyte character string stored at s is not correct");
      }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
 
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the wide character sequence starting at pwcs has a wide character which
   cannot be translated to multibyte character in the translation range,
   verify this function returns -1. */
private void
test3()
{
  size_t ret;
  unsigned char mbs[MAX_BYTE_LEN];
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character sequence starting at pwcs has a wide "
	       "character which cannot be translated to multibyte character "
	       "in the translation range, verify this function returns -1.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = wcstombs (mbs, incorrect_wstr, length_inc_str)) == -1)
    PATH_TRACE;
  else
    {
      PATH_TRACE;
      xx_rpt (FAILURE);
      in_rpt ("The return value is incorrect.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
