/*
 * LI18NUX2K.L1/base/wscanf/wscanf.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <errno.h>
#include <locale.h>
#include <wchar.h>

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();
private void test16();
private void test17();
private void test18();
private void test19();
private void test20();
private void test21();
private void test22();
private void test23();
private void test24();

#define NO_TESTS 24
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,   1,
	test2,   2,
	test3,   3,
	test4,   4,
	test5,   5,
	test6,   6,
	test7,   7,
	test8,   8,
	test9,   9,
	test10, 10,
	test11, 11,
	test12, 12,
	test13, 13,
	test14, 14,
	test15, 15,
	test16, 16,
	test17, 17,
	test18, 18,
	test19, 19,
	test20, 20,
	test21, 21,
	test22, 22,
	test23, 23,
	test24, 24,
	NULL,    0
};


/* Compare two arrays of wide characters.
   If two array is the same, this function returns 1, others returns 0. */
private int
is_wcs_same (const wchar_t *ws1, const wchar_t *ws2, size_t n)
{
  int i;

  for (i = 0; i < n; i++)
    if (ws1[i] != ws2[i])
      return 0;

  return 1;
}


/* Verify a directive composed one or more whitespace wide character is
   executed by reading input until no more valid input can be read, or up
   to the first wide character which is not whitespace wide character,
   which remains unread. */
private void
test1()
{
  wchar_t wc;
  int ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Verify a directive composed one or more whitespace wide "
	       "character is executed by reading input until no more valid "
	       "input can be read, or up to the first wide character which "
	       "is not whitespace wide character, which remains unread.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile1", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile1'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "  a \t\t ");
  fclose (fp);

  if ((fp = freopen ("testfile1", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile1'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"  %lc", &wc);
  if (wc == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Did not skip up to the first wide character which is not "
	      "white space wide character.");
      DBUG_PRINTF ("test1", "Character: %lc", wc);
    }

  ret = wscanf (L" \t%lc", &wc);
  if (ret == EOF)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Did not skip no more valid input can be read.");
      DBUG_PRINTF ("test1", "Character: %lc", wc);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If a directive is an ordinary wide character (i.e. except for %, and
   whitespace wide character), verify the next character is read, and 
   compare with the directive and the read one and when they are not
   equivalent, the directive fails, and the differing and subsequent wide
  characters remain unread. */
private void
test2()
{
  wchar_t wc;
  int ret, n;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a directive is an ordinary wide character (i.e. except "
	       "for %, and whitespace wide character), verify the next "
	       "character is read, and compare with the directive and the "
	       "read one and when they are not equivalent, the directive "
	       "fails, and the differing and subsequent wide characters "
	       "remain unread.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile2", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile2'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "a9bc8");
  fclose (fp);

  if ((fp = freopen ("testfile2", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile2'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"a%d", &n);
  if (n == 9)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The next character is not read.");
      DBUG_PRINTF ("test2", "number: %d", n);
    }

  ret = wscanf (L"bb%d", &n);
  if (ret < 1)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Subsequent wide characters does not remain unread.");
      DBUG_PRINTF ("test2", "Return Value: %d", ret);
    }

  ret = wscanf (L"c%d", &n);
  if (n == 8)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The next character is not read.");
      DBUG_PRINTF ("test2", "number: %d", n);
    }

  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}


/* If a conversion specification has `*', verify the result of conversion is
   not stored any arguments. */
private void
test3()
{
  wchar_t wc;
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specification has `*', verify the result of "
	       "conversion is not stored any arguments.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile3", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile3'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "ab");
  fclose (fp);

  if ((fp = freopen ("testfile3", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile3'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%*lc%lc", &wc);
  if (wc == L'b')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("`*' is interpreted wrongly.");
      DBUG_PRINTF ("test3", "Return Value: %d", ret);
      DBUG_PRINTF ("test3", "Read: %lc", wc);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion specification has a field width, verify this function
   interprets the interpreted wide character in the number of the field
   width. */
private void
test4()
{
  wchar_t wcs[] = {L'\0', L'\0', L'\0', L'\0', L'\0',
		  L'\0', L'\0', L'\0', L'\0', L'\0'};
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion specification has a field width, verify this "
	       "function interprets the interpreted wide character in the "
	       "number of the field width.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile4", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile4'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "123456789");
  fclose (fp);

  if ((fp = freopen ("testfile4", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile4'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%5lc", wcs);
  if (is_wcs_same (wcs, L"12345\0", 6))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The field width is interpreted wrongly.");
      DBUG_PRINTF ("test4", "Return Value: %d", ret);
      DBUG_PRINTF ("test4", "Read: %ls", wcs);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `d', verify the conversion result is the
   same as the results of wcstol() with the value 10 for the base argument. */
private void
test5()
{
  wchar_t wcs[] = L"   123";
  char    str[] =  "   123";
  long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `d', verify the conversion "
	       "result is the same as the results of wcstol() with the value "
	       "10 for the base argument.";

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile5", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile5'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile5", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile5'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%ld", &n1);
  n2 = wcstol(wcs, NULL, 10);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `d' is not interpreted correctly");
      in_rpt ("(Or wcstol() is something wrong.)");
      DBUG_PRINTF ("test5", "Return Value: %d", ret);
      DBUG_PRINTF ("test5", "wscanf(): %ld, wcstol(): %ld", n1, n2);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `i', verify the conversion result is the
   same as the results of wcstol() with the value 0 for the base argument. */
#define TEST6_STR_NUM	4
private void
test6()
{
  wchar_t *wcs[] = {L"   077", L" 123", L"  0xff", L"  0XFFFF"};
  char    *str[] = { "   077",  " 123",  "  0xff",  "  0xFFFF"};
  int  i, ret;
  long n1, n2;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `i', verify the conversion "
	       "result is the same as the results of wcstol() with the value "
	       "0 for the base argument.";

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }
  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile6", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile6'");
      DBUG_VOID_RETURN;
    }
  for (i = 0; i < TEST6_STR_NUM; i++)
    fprintf (fp, "%s\n", str[i]);
  fclose (fp);

  if ((fp = freopen ("testfile6", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile6'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < TEST6_STR_NUM; i++)
    {
      ret = wscanf (L"%li", &n1);
      n2 = wcstol(wcs[i], NULL, 0);

      if (n1 == n2)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Conversion wide character `i' is not interpreted correctly");
	  in_rpt ("(Or wcstol() is something wrong.)");
	  DBUG_PRINTF ("test6", "Return Value: %d", ret);
	  DBUG_PRINTF ("test6", "wscanf(): %li, wcstol(): %li", n1, n2);
	}
    }

  PATH_XS_RPT(TEST6_STR_NUM);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `o', verify the conversion result is the
   same as the results of wcstoul() with the value 8 for the base argument. */
private void
test7()
{
  wchar_t wcs[] = L"   779";
  char    str[] =  "   779";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `o', verify the conversion "
	       "result is the same as the results of wcstoul() with the value "
	       "8 for the base argument.";

  DBUG_ENTER ("test7");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile7", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile7'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile7", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile7'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%lo", &n1);
  n2 = wcstoul(wcs, NULL, 8);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `o' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test7", "Return Value: %d", ret);
      DBUG_PRINTF ("test7", "wscanf(): %lo, wcstoul(): %lo", n1, n2);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `u', verify the conversion result is the
   same as the results of wcstoul() with the value 10 for the base argument. */
private void
test8()
{
  wchar_t wcs[] = L"   2147483648";
  char    str[] =  "   2147483648 ";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `u', verify the conversion "
	       "result is the same as the results of wcstoul() with the value "
	       "10 for the base argument.";

  DBUG_ENTER ("test8");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile8", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile8'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile8", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile8'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%lu", &n1);
  n2  = wcstoul(wcs, NULL, 10);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `u' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test8", "Return Value: %d", ret);
      DBUG_PRINTF ("test8", "wscanf(): %lu, wcstoul(): %lu", n1, n2);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `x' or `X', verify the conversion
   result is the same as the results of wcstoul() with the value 16 for the
   base argument. */
private void
test9()
{
  wchar_t wcs[] = L"   ABCDEFG";
  char    str[] =  "   ABCDEFG ";
  unsigned long n1, n2;
		 
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `x' or `X', verify the "
	       "conversion result is the same as the results of wcstoul() "
	       "with the value 16 for the base argument.";

  DBUG_ENTER ("test9");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile9", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile9'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile9", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile9'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%lx", &n1);
  n2 = wcstoul(wcs, NULL, 16);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `x' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test9", "Return Value: %d", ret);
      DBUG_PRINTF ("test9", "wscanf(): %lx, wcstoul(): %lx", n1, n2);
    }
  rewind (fp);

  ret = wscanf (L"%lX", &n1);
  n2 = wcstoul(wcs, NULL, 16);

  if (n1 == n2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character `X' is not interpreted correctly");
      in_rpt ("(Or wcstoul() is something wrong.)");
      DBUG_PRINTF ("test9", "Return Value: %d", ret);
      DBUG_PRINTF ("test9", "wscanf(): %lX, wcstoul(): %lX", n1, n2);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `e', `E', `f', `g' or `G', verify the
   conversion result is the same as the results of wcstod(). Especially
   check the radix character according to the current locale is interpreted
   correctly. */
private void
test10()
{
  wchar_t *conv[] = {L"%le", L"%lE", L"%lf", L"%lg", L"%lG"};
  wchar_t wcs[] = L"   9*9*9";
  char    str[] =  "   9*9*9";
  double  d1, d2;
  int  i, ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `e', `E', `f', `g' or `G', "
	       "verify the conversion result is the same as the results of "
	       "wcstod(). Especially check the radix character according to "
	       "the current locale is interpreted correctly.";

  DBUG_ENTER ("test10");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile10", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile10'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = fopen ("dummyfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'dummyfile'");
      DBUG_VOID_RETURN;
    }
  fclose (fp);

 for (i = 0; i < sizeof (conv) / sizeof (conv[0]); i++)
   {
     if ((fp = freopen ("dummyfile", "r", stdin)) == NULL)
       {
	 xx_rpt (FAILURE);
	 in_rpt ("Cannot open 'dummyfile'");
	 DBUG_VOID_RETURN;
       }

     if ((fp = freopen ("testfile10", "r", stdin)) == NULL)
       {
	 xx_rpt (FAILURE);
	 in_rpt ("Cannot open 'testfile10'");
	 DBUG_VOID_RETURN;
       }

     ret = wscanf (conv[i], &d1);
     d2 = wcstod(wcs, NULL);

     if (d1 == d2)
       PATH_TRACE;
     else
       {
	 xx_rpt (FAILURE);
	 in_rpt ("Conversion wide character is not interpreted correctly");
	 in_rpt ("(Or wcstoul() is something wrong.)");
	 DBUG_PRINTF ("test10", "Conversion character: %ls", conv[i]);
	 DBUG_PRINTF ("test10", "Return Value: %d", ret);
	 DBUG_PRINTF ("test10", "wscanf(): %lf, wcstoul(): %lf", d1, d2);
       }
   }

  PATH_XS_RPT(sizeof(conv) / sizeof(conv[0]));
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `c' and conversion specification does
   not have `l', verify this function reads a sequence of the wide
   characters of the number specified by the field width, and stores the
   multibyte character sequence as same that are made of the sequence of
   the wide characters by wcsrtombs() with initial conversion state. */
private void
test11()
{
  wchar_t wcs[] = {L'A', 0x00C4, 0x3042, L'\0'}, *p;
  const wchar_t **pp;
  char *str1 = "AÄあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `c' and conversion "
	       "specification does not have `l', verify this function reads "
	       "a sequence of the wide characters of the number specified by "
	       "the field width, and stores the multibyte character sequence "
	       "as same that are made of the sequence of the wide characters "
	       "by wcsrtombs() with initial conversion state.";

  DBUG_ENTER ("test11");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile11", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile11'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = freopen ("testfile11", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile11'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%4c", &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, bytelen) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `c' and conversion specification has `l',
   verify this function reads a sequence of the wide characters of the number
   specified by the field width, and stores them. */
private void
test12()
{
  wchar_t wcs1[] = {L'A', 0x00C4, 0x3042, L'\0'}, wcs2[BUFSIZ];
  char *str = "AÄあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `c' and conversion "
	       "specification has `l', verify this function reads a sequence "
	       "of the wide characters of the number specified by the field "
	       "width, and stores them.";

  DBUG_ENTER ("test12");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile12", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile12'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile12", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile12'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%4lc", &wcs2);

  if (is_wcs_same (wcs1, wcs2, 4))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `s' and conversion specification does
   not have `l', verify this function reads a wide character string, and
   stores the multibyte character string as same that are made of the wide
   character string by wcsrtombs() with initial conversion state.
   Especially check the null character is stood at the end of the multibyte
   character string. */
private void
test13()
{
  wchar_t wcs[] = {L'A', 0x00C4, 0x3042, L'\0'}, *p;
  const wchar_t **pp;
  char *str1 = "   AÄあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `s' and conversion "
	       "specification does not have `l', verify this function reads "
	       "a wide character string, and stores the multibyte character "
	       "string as same that are made of the wide character string by "
	       "wcsrtombs() with initial conversion state. Especially check "
	       "the null character is stood at the end of the multibyte "
	       "character string.";

  DBUG_ENTER ("test13");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile13", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile13'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = freopen ("testfile13", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile13'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%s", &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, bytelen) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `s' and conversion specification has
   `l', verify this function reads a wide character string, and stores
   them. Especially check the null wide character is stood at the end of
   wide character string. */
private void
test14()
{
  wchar_t wcs1[] = {L'A', 0x00C4, 0x3042, L'\0'}, wcs2[BUFSIZ];
  char *str = "AÄあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `s' and conversion "
	       "specification has `l', verify this function reads a wide "
	       "character string, and stores them. Especially check the null "
	       "wide character is stood at the end of wide character string.";

  DBUG_ENTER ("test14");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile14", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile14'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile14", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile14'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (L"%ls", &wcs2);

  if (is_wcs_same (wcs1, wcs2, 4))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide characters are `[...]', verify this function reads
   the wide characters which are included between `[' and ']', and stores
   the multibyte character sequence as same that are made of the sequence
   of the wide characters by wcsrtombs() with initial conversion state.
   Especially check the null character is stood at the end of the multibyte
   character string. */
private void
test15()
{
  wchar_t wcs[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'};
  wchar_t conv[] = {L'%', L'[', L'A', 0x00C4, 0x3042, L']', L'\0'};
  wchar_t *p;
  const wchar_t **pp;
  char *str1 = "AÄAああいあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide characters are `[...]', verify this "
	       "function reads the wide characters which are included "
	       "between `[' and ']', and stores the multibyte character "
	       "sequence as same that are made of the sequence of the wide "
	       "characters by wcsrtombs() with initial conversion state. "
	       "Especially check the null character is stood at the end of "
	       "the multibyte character string.";

  DBUG_ENTER ("test15");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile15", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile15'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = freopen ("testfile15", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile15'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (conv, &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);

  if (memcmp (str2, str3, (bytelen + 1)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `l[...]', verify this function reads
   the wide characters which are included between `[' and ']', and stores
   them. Especially check the null wide character is stood at the end of
   the wide character string. */
private void
test16()
{
  wchar_t wcs1[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'}, wcs2[BUFSIZ];
  wchar_t conv[] = {L'%', L'l', L'[', L'A', 0x00C4, 0x3042, L']', L'\0'};
  char *str = "AÄAああいあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `l[...]', verify this "
	       "function reads the wide characters which are included "
	       "between `[' and ']', and stores them. Especially check the "
	       "null wide character is stood at the end of the wide "
	       "character string.";

  DBUG_ENTER ("test16");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile16", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile16'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile16", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile16'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (conv, &wcs2);
  DBUG_PRINTF ("test16", "wcs1: %ls, wcs2: %ls", wcs1, wcs2);

  if (is_wcs_same (wcs1, wcs2, sizeof(wcs1) / sizeof(wcs1[0])))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `[^...]', verify this function reads
   the wide characters which are not included between `[^' and ']', and
   stores the multibyte character sequence as same that are made of the
   sequence of the wide characters by wcsrtombs() with initial conversion
   state. Especially check the null character is stood at the end of the
   multibyte character string. */
private void
test17()
{
  wchar_t wcs[]  = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'};
  wchar_t conv[] = {L'%', L'[', L'^', L'B', 0x00FC, 0x3044, L']', L'\0'};
  wchar_t *p;
  const wchar_t **pp;
  char *str1 = "AÄAああいあ", str2[BUFSIZ], str3[BUFSIZ];
  mbstate_t state;
  size_t bytelen;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `[^...]', verify this "
	       "function reads the wide characters which are not included "
	       "between `[^' and ']', and stores the multibyte character "
	       "sequence as same that are made of the sequence of the wide "
	       "characters by wcsrtombs() with initial conversion state. "
	       "Especially check the null character is stood at the end of "
	       "the multibyte character string.";

  DBUG_ENTER ("test17");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile17", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile17'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str1);
  fclose (fp);

  if ((fp = freopen ("testfile17", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile17'");
      DBUG_VOID_RETURN;
    }

  ret = wscanf (conv, &str2);
  memset (&state, '\0', sizeof(mbstate_t));
  p =  wcs;
  pp = (const wchar_t **)&p;
  bytelen = wcsrtombs (str3, pp, BUFSIZ, &state);
  DBUG_PRINTF ("test18", "str2: %s, str3: %s", str2, str3);

  if (memcmp (str2, str3, (bytelen + 1)) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `l[^...]', verify this function reads
   a sequence of the wide characters which is not included between `[^' and
   ']', and stores them. Especially check the null character is stood at
   the end of the multibyte character string. */
private void
test18()
{
  wchar_t wcs1[] = {L'A', 0x00C4, L'A', 0x3042, 0x3042, L'\0'}, wcs2[BUFSIZ];
  wchar_t conv[] = {L'%', L'l', L'[', L'^', L'B', 0x00FC, 0x3044, L']', L'\0'};
  char *str = "AÄAああいあ";
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `l[^...]', verify this "
	       "function reads a sequence of the wide characters which is "
	       "not included between `[^' and ']', and stores them. "
	       "Especially check the null character is stood at the end of "
	       "the multibyte character string.";

  DBUG_ENTER ("test18");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile18", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile18'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile18", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile18'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (conv, &wcs2);
  DBUG_PRINTF ("test18", "wcs1: %ls, wcs2: %ls", wcs1, wcs2);

  if (is_wcs_same (wcs1, wcs2, sizeof(wcs1) / sizeof(wcs1[0])))
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character is not interpreted correctly");
      in_rpt ("(Or wcstombs() is something wrong.)");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `p', verify this function stores the
   pointer value. */
private void
test19()
{
  char a = 4;
  void *p1, *p2;
  FILE *fp;
  int  ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `p', verify this function "
	       "stores the pointer value.";

  DBUG_ENTER ("test19");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile19", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile19'");
      DBUG_VOID_RETURN;
    }
  p1 = &a;
  fprintf (fp, "%p", p1);
  fclose (fp);

  if ((fp = freopen ("testfile19", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile19'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%p", &p2);

  if (*(char *)p2 == 4)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character p is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `n', verify no input is consumed, and
   this function stores the number of the wide character which the function
   has read. */
#define TEST20_WCHAR_NUM 7
private void
test20()
{
  char *str = "AÄAああいあ";
  wchar_t wc1, wc2, wcs[BUFSIZ];
  FILE *fp;
  int  n, ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `n', verify no input is "
	       "consumed, and this function stores the number of the wide "
	       "character which the function has read.";

  DBUG_ENTER ("test20");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile20", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile20'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%s", str);
  fclose (fp);

  if ((fp = freopen ("testfile20", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile20'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%lc%lcA%ls%n", &wc1, &wc2, wcs, &n);

  if (n == TEST20_WCHAR_NUM)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character n is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If a conversion wide character is `%' (i.e. the orthography is `%%', it
   matched a wide character %. */ 
private void
test21()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If a conversion wide character is `%' (i.e. the orthography "
	       "is `%%', it matched a wide character %.";

  DBUG_ENTER ("test21");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile21", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile21'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "%%9");
  fclose (fp);

  if ((fp = freopen ("testfile21", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile21'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%%%d", &n);

  if (n == 9)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Conversion wide character % is not interpreted correctly");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If this function has not did any conversion by a read error, verify this
   function returns EOF. */
private void
test22()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If this function has not did any conversion by a read error, "
	       "verify this function returns EOF.";

  DBUG_ENTER ("test22");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile22", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile22'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "");
  fclose (fp);

  if ((fp = freopen ("testfile22", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile22'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%d", &n);

  if (ret == EOF)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test22", "Return Value: %d", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If this function has did one or more conversion, verify this function
   returns the number of the conversion. */ 
private void
test23()
{
  FILE *fp;
  int  ret, n;
  char str[BUFSIZ];
  wchar_t wc;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If this function has did one or more conversion, verify this "
	       "function returns the number of the conversion.";

  DBUG_ENTER ("test23");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile23", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile23'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "123abAあいう");
  fclose (fp);

  if ((fp = freopen ("testfile23", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile23'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%dab%lc%s", &n, &wc, str);

  if (ret == 3)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test23", "Return Value: %d", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* After executing this function correctly, verify standard input is
   changed to wide character oriented. */
private void
test24()
{
  FILE *fp;
  int  ret, n;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "After executing this function correctly, verify standard "
	       "input is changed to wide character oriented.";

  DBUG_ENTER ("test24");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  if ((fp = fopen ("testfile24", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile24'");
      DBUG_VOID_RETURN;
    }
  fprintf (fp, "9abc");
  fclose (fp);

  if ((fp = freopen ("testfile24", "r", stdin)) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile24'");
      DBUG_VOID_RETURN;
    }
  
  ret = wscanf (L"%d", &n);

  if (fwide (stdin, 0) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
