#/bin/sh
#
# LI18NUX2K.L1/utils/join/join.sh
#
# Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
# (X/Open)
# trading as The Open Group
# Copyright (c) 2001, 2003 International Business Machines Corp.
# All rights reserved except as granted by this License.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the "Artistic License" which comes with this
# Kit, with the following modification:
# a) "executable(s)" should be interpreted to include
# "test case(s)"
# b) if you wish to make changes as defined in clause 2 and 3, and
# distribute a modified version of this package, then
# clauses 3c and 4c are required
# c) Clause 7 is rephrased as follows: "Subroutines supplied by you
# and linked into this Package shall not be considered part of this
# Package".
#
#
# X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
# LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# You should have received a copy of the Artistic License with this
# Kit, in the file named "Artistic".  If not, we'll be glad to provide one.


tet_startup="startup"			# startup function
tet_cleanup="cleanup"			# cleanup function
iclist="ic1"
ic1="tp1 tp2 tp3 tp4"

tp1()
{
	export LANG="LTP_1.UTF-8"
	tet_infoline "* Verify this utility joins files and writes the results correctly, when these files is ordered according to the collating sequence of the current locale."
	tet_infoline " "

	join -t ' ' -a 1 -a 2 -e '(null)' -o 0,1.2,2.2 tp1_a.input tp1_b.input > tp1.output
	diff tp1.correct tp1.output > /dev/null 2>&1

	if [ $? = 0 ] ; then
		tet_result PASS # success
	else
		tet_infoline "This utility does not join the files."
		tet_result FAIL # fail
	fi
	unset LANG
}

tp2()
{
	export LANG="LTP_1.UTF-8"
	tet_infoline "* When -e option is specified, verify this utility replaces empty output fields in the list selected by -o with the string even if the string is written in multibyte characters."
	tet_infoline " "

	join -t ' ' -a 1 -a 2 -e 'なし' -o 0,1.2,2.2 tp1_a.input tp1_b.input > tp2.output
	diff tp2.correct tp2.output > /dev/null 2>&1

	if [ $? = 0 ] ; then
		tet_result PASS # success
	else
		tet_infoline "This utility does not join the files."
		tet_result FAIL # fail
	fi
	unset LANG
}

tp3()
{
	export LANG="LTP_1.UTF-8"
	tet_infoline "* When -t option is specified, verify this utility uses given character as a separator even if the character is multibyte character."
	tet_infoline " "

	join -t '＿' -a 1 -a 2 -e '(null)' -o 0,1.2,2.2 tp3_a.input tp3_b.input > tp3.output
	diff tp3.correct tp3.output > /dev/null 2>&1

	if [ $? = 0 ] ; then
		tet_result PASS # success
	else
		tet_infoline "This utility does not join the files."
		tet_result FAIL # fail
	fi
	unset LANG
}

tp4()
{
	export LANG="LTP_1.UTF-8"
	tet_infoline "* When -t option is not specified, verify blank characters according to the current locale are used as input field separators."
	tet_infoline " "

	join -a 1 -a 2 -e '(null)' -o 0,1.2,2.2 tp4_a.input tp4_b.input > tp4.output
	diff tp4.correct tp4.output > /dev/null 2>&1

	if [ $? = 0 ] ; then
		tet_result PASS # success
	else
		tet_infoline "This utility does not join the files."
		tet_result FAIL # fail
	fi
	unset LANG
}

startup() # startup function
{
          rm -f out.stdout out.stderr > /dev/null 2>&1
}

cleanup() # clean-up function
{
          rm -f out.stdout out.stderr > /dev/null 2>&1
}

# source common shell functions

. $TET_EXECUTE/LIB/i18nfuncs.sh
. $TET_EXECUTE/LIB/shfuncs.sh
. $TET_EXECUTE/LIB/lsbfuncs.sh

# execute shell test case manager - must be last line
. $TET_ROOT/lib/xpg3sh/tcm.sh

