/*
 * Copyright (c) 2002-2003 WIPRO TECHNOLOGIES 
 * Developed for LSB-PAM project by WIPRO TECHNOLOGIES
 */

/*
 * Project: LSB-PAM
 *
 * File: tset/LSB.pam/testcases/pam_end/pam_end.c
 * This file contains the test cases for testing the interface pam_end.
 *
 * Author(s) : Rohit R <rohit.raveendran@wipro.com>, WIPRO TECHNOLOGIES
 *             Laavanya K R <laavanya.gopalan@wipro.com>, WIPRO TECHNOLOGIES
 *
 * Modifications:
 *
 */

#include <stdio.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

#include <std.h>
#include <tet_api.h>

/*******************************************************************/

#define NUM_TESTS   2 /* To be updated when number of tests are modified */

private void tc1();
private void tc2();

private void prepare_tests();
private void clean_tests();

public  void (*tet_startup)() = prepare_tests;
public  void (*tet_cleanup)() = clean_tests;

private char *conf_file;
private char *user;

private char echo[256];

private pam_handle_t *pamh;
private struct pam_conv pamc;

public struct tet_testlist tet_testlist[] = {
        { tc1, 1 },
        { tc2, 2 },
        { NULL, 0 }
};

private void delete_tests(char *msg);

/*******************************************************************/

private void
prepare_tests()
{
    int    retval;
 
    tet_infoline(" starting tests pam_end ");

    pamh = NULL;
    pamc.conv = &misc_conv;
    pamc.appdata_ptr = NULL;

    conf_file = tet_getvar("LSB_PAM_CONF_FILE");

    if (conf_file == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_CONF_FILE unset");
        return;
    }

    user = tet_getvar("LSB_PAM_USER");

    if (user == NULL) {
        delete_tests(" tetexec.cfg parameter LSB_PAM_USER unset");
        return;
    }

    return;
}


private void
clean_tests()
{
    if (pamh) {
        pam_end(pamh, 0);
    }

    tet_infoline(" end of tests pam_end");
    return;
}


/*
 * tc1() :
 *     This test case checks the interface pam_end with valid
 *     parameters. The expected return value is PAM_SUCCESS.
 */

private void
tc1()
{
    int    retval;

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_end(pamh, 0);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_end returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    pamh = NULL;
    return;
}


/*
 * tc2() :
 *     This test case checks the interface pam_end with NULL
 *     for the pam handle. pam_end is not expected to return PAM_SUCCESS.
 */

private void
tc2()
{
    int    retval;

    retval = pam_start(conf_file, user, &pamc, &pamh);

    if (retval != PAM_SUCCESS) {
        sprintf(echo," pam_start returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        delete_tests(echo);
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_end(NULL, 0);

    if (retval == PAM_SUCCESS) {
        tet_infoline(" pam_end returned PAM_SUCEESS, when expected to fail");
        tet_result(TET_FAIL);
        return;
    }

    retval = pam_end(pamh, 0);

    if (retval != PAM_SUCCESS) {
        tet_printf(" pam_end returned %s when expected to succeed",
                                pam_strerror(NULL, retval));
        tet_result(TET_FAIL);
        return;
    }

    tet_result(TET_PASS);
    pamh = NULL;
    return;
}



/*
 *  delete_tests():
 *       This routine is called to stop execution of the test cases
 *       in case of initialization errors.
 */

private void
delete_tests(char *msg)
{
    int    count;

    for (count = 1; count <= NUM_TESTS; count++) {
        tet_delete(count, msg);
    }

    return;
}
